<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Route
 * Набор функций для работы с ЧПУ
 */
class Route
{
	/**
	 * @var object основной объект системы
	 */
	private $diafan;

	/**
	 * @var array кэш класса
	 */
	private $cache;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Генерирует ссылку
	 *
	 * @param integer $site_id номер страницы сайта
	 * @param string $module_name модуль
	 * @param integer $cat_id номер категории
	 * @param integer $element_id номер элемента модуля
	 * @param integer $param_id номер параметра
	 * @return string
	 */
	public function link($site_id, $module_name = '', $cat_id = 0, $element_id = 0, $param_id = 0)
	{
		$link = '';
		if ($param_id)
		{
			if ($this->get_rewrite($module_name, $param_id, "param"))
			{
				$link = $this->get_rewrite($module_name, $param_id, "param").ROUTE_END;
			}
			else
			{
				$link = ($this->get_rewrite("site", $site_id) ? $this->get_rewrite("site", $site_id).'/' : '')
				        .'param'.$param_id.'/';
			}
		}
		elseif ($element_id)
		{
			if ($this->get_rewrite($module_name, $element_id, "element"))
			{
				$link = $this->get_rewrite($module_name, $element_id, "element").ROUTE_END;
			}
			elseif ($cat_id && $this->get_rewrite($module_name, $cat_id, "cat"))
			{
				$link = $this->get_rewrite($module_name, $cat_id, "cat").'/show'.$element_id.'/';
			}
			else
			{
				$link = ($this->get_rewrite("site", $site_id) ? $this->get_rewrite("site", $site_id).'/' : '')
				        .($cat_id ? 'cat'.$cat_id.'/' : '')
				        .'show'.$element_id.'/';
			}
		}
		elseif ($cat_id)
		{
			if ($this->get_rewrite($module_name, $cat_id, "cat"))
			{
				$link = $this->get_rewrite($module_name, $cat_id, "cat").ROUTE_END;
			}
			else
			{
				$link = ($this->get_rewrite("site", $site_id) ? $this->get_rewrite("site", $site_id).'/' : '')
				        .'cat'.$cat_id.'/';
			}
		}
		else
		{
			$link = $this->get_rewrite("site", $site_id) ? $this->get_rewrite("site", $site_id).($module_name ? '/' : ROUTE_END) : '';
		}
		return $link;
	}

	/**
	 * Получает ЧПУ страницы сайта по названию модуля
	 * 
	 * @param string $module_name название модуля
	 * @return string
	 */
	public function module($module_name,$route_end=FALSE)
	{
		//if (! isset($this->cache["page_addmodule"][$module_name]))
		//{
			$id = DB::query_result("SELECT id FROM {site} WHERE addmodule='%h' AND trash='0' AND [act]='1'"
			                       .(! $this->diafan->_user->id ? " AND access='0'" : ''), $module_name);
			$this->cache["page_addmodule"][$module_name] = $this->link($id,$route_end ? '' : $module_name);
		//}
		return $this->cache["page_addmodule"][$module_name];
	}

	/**
	 * Выдает URL текущей страницы с включенными или исключенными переменными
	 * 
	 * @param string|array $exclude исключенные переменные
	 * @param array $include включенные переменные
	 * @return string
	 */
	public function current_link($exclude = '', $include = array())
	{
		if (! is_array($exclude))
		{
			$exclude = array($exclude);
		}
		$args = array();
		$keys = array();
		foreach ($this->diafan->rewrite_variable_names_all as $arg)
		{
			if (in_array($arg, array_keys($include)))
			{
				$args[] = array($arg => $include[$arg]);
				$keys[] = $arg;
			}
			elseif (! in_array($arg, $exclude))
			{
				if (! empty($this->diafan->$arg) && ($arg != "page" || $this->diafan->page != 1) && ($arg != "comments" || $this->diafan->comments != 1))
				{
					$args[] = array($arg => $this->diafan->$arg);
					$keys[] = $arg;
				}
			}
		}
		if (! $args)
		{
			$link = $this->link($this->diafan->cid);
		}
		elseif (in_array('show', $keys) && $this->get_rewrite($this->diafan->module, $this->diafan->show, "element"))
		{
			foreach ($args as $i => $array)
			{
				if (! empty($array["show"]) || ! empty($array["cat"]))
				{
					unset($args[$i]);
				}
			}
			if (! $args)
			{
				$link = $this->get_rewrite($this->diafan->module, $this->diafan->show, "element").ROUTE_END;
			}
			else
			{
				$link = $this->get_rewrite($this->diafan->module, $this->diafan->show, "element").'/';
			}
		}
		elseif (in_array('cat', $keys) && $this->get_rewrite($this->diafan->module, $this->diafan->cat, "cat"))
		{
			foreach ($args as $i => $array)
			{
				if (! empty($array["cat"]))
				{
					unset($args[$i]);
				}
			}
			if (! $args)
			{
				$link = $this->get_rewrite($this->diafan->module, $this->diafan->cat, "cat").ROUTE_END;
			}
			else
			{
				$link = $this->get_rewrite($this->diafan->module, $this->diafan->cat, "cat").'/';
			}
		}
		elseif (in_array('param', $keys) && $this->get_rewrite($this->diafan->module, $this->diafan->param, "param"))
		{
			foreach ($args as $i => $array)
			{
				if (! empty($array["param"]))
				{
					unset($args[$i]);
				}
			}
			if (! $args)
			{
				$link = $this->get_rewrite($this->diafan->module, $this->diafan->param, "param").ROUTE_END;
			}
			else
			{
				$link = $this->get_rewrite($this->diafan->module, $this->diafan->param, "param").'/';
			}
		}
		else
		{
			$link = $this->get_rewrite("site", $this->diafan->cid) ? $this->get_rewrite("site", $this->diafan->cid).'/' : '';
		}
		foreach ($args as $array)
		{
			foreach ($array as $name => $value)
			{
				$link .= $name.$value.'/';
			}
		}
		return $link;
	}

	/**
	 * Выдает URL текущей страницы административной части с включенными переменными
	 * 
	 * @param string|array $exclude исключенные переменные
	 * @return string
	 */
	public function current_admin_link($exclude = '')
	{
		$key = serialize($exclude);
		if (! empty($this->cache[$key]))
		{
			return $this->cache[$key];
		}
		if (! is_array($exclude))
		{
			$exclude = array($exclude);
		}
		$args = array();
		foreach ($this->diafan->rewrite_variable_names as $arg)
		{
			if (! in_array($arg, $exclude))
			{
				if (! empty($this->diafan->$arg) && ($arg != "page" || $this->diafan->page != 1))
				{
					$args[] = array($arg => $this->diafan->$arg);
				}
			}
		}
		$link = BASE_PATH_HREF.($this->diafan->rewrite ? $this->diafan->rewrite.'/' : '');
		foreach ($args as $array)
		{
			foreach ($array as $name => $value)
			{
				$link .= $name.$value.'/';
			}
		}
		$this->cache[$key] = $link;
		return $link;
	}

	/**
	 * Ищет псевдоссылку в базе данных
	 *
	 * @param string $rewrite текущая псевдоссылка
	 * @param boolean $arguments_in_url в URL переданы аргументы
	 * @return array|boolean false
	 */
	public function search($rewrite, $arguments_in_url = FALSE)
	{
		if (ROUTE_END != "/" && ! $arguments_in_url)
		{
			if (preg_match('/(.*)'.ROUTE_END.'$/', $rewrite, $match))
			{
				$rewrite = $match[1];
			}
			else
			{
				return FALSE;
			}
		}
		if ($row = DB::fetch_array(DB::query("SELECT site_id, module_name, cat_id, element_id, param_id, rewrite FROM {rewrite} WHERE trash='0' AND rewrite='%h'", $rewrite)))
		{
			return $row;
		}
		return FALSE;
	}

	/**
	 * Получает ЧПУ по тегу
	 *
	 * @param string $module_name модуль
	 * @param integer $id номер элемента
	 * @param string $tag тег
	 * @return boolean true
	 */
	private function get_rewrite($module_name, $id, $tag = "")
	{
		if (isset($this->cache["rewrites"][$module_name.'_'.$tag.$id]))
		{
			return $this->cache["rewrites"][$module_name.'_'.$tag.$id];
		}
		if ($module_name == 'site')
		{
			$where = "site_id=%d";
		}
		else
		{
			switch($tag)
			{
				case 'param':
					$where = "param_id=%d";
					break;
				case 'cat':
					$where = "cat_id=%d";
					break;
				default:
					$where = "element_id=%d";
					break;
			}
		}
		$this->cache["rewrites"][$module_name.'_'.$tag.$id] = DB::query_result("SELECT rewrite FROM {rewrite} WHERE trash='0' AND module_name='%h' AND ".$where, $module_name, $id);

		return $this->cache["rewrites"][$module_name.'_'.$tag.$id];
	}
}
