<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(__FILE__)).'includes/404.php');
}

/**
 * Vaidate
 *
 * Класс для валидации данных
 */
class Validate
{
	/**
	 * Проверяет логин на валидность
	 *
	 * @param string $name логин
	 * @return string|boolean false
	*/
	public static function login($name)
	{
		if (! strlen($name))
			return _LANG_ENTER_USERNAME;

		if (substr($name, 0, 1) == ' ')
			return _LANG_LTRIM_LOGIN;

		if (substr($name, -1) == ' ')
			return _LANG_RTRIM_LOGIN;

		if (strpos($name, '  ') !== FALSE)
			return _LANG_LOGIN_NOT_SPACE;

		if (preg_match("/[^\x80-\xF7 [:alnum:]@_.-]/", $name))
			return _LANG_LOGIN_INCORRECT_CHARS;

		if (preg_match('/[\x{80}-\x{A0}'         // Non-printable ISO-8859-1 + NBSP
		               .'\x{AD}'                 // Soft-hyphen
		               .'\x{2000}-\x{200F}'      // Various space characters
		               .'\x{2028}-\x{202F}'      // Bidirectional text overrides
		               .'\x{205F}-\x{206F}'      // Various text hinting characters
		               .'\x{FEFF}'               // Byte order mark
		               .'\x{FF01}-\x{FF60}'      // Full-width latin
		               .'\x{FFF9}-\x{FFFD}'      // Replacement characters
		               .'\x{0}]/u'               // NULL byte
		               ,$name))
			return _LANG_LOGIN_INCORRECT_CHARS;

		if (strpos($name, '@') !== FALSE && !preg_match('/@([0-9a-z](-?[0-9a-z])*.)+[a-z]{2}([zmuvtg]|fo|me)?$/', $name))
			return _LANG_LOGIN_INCORRECT;

		if (strlen($name) > 15)
			return _LANG_LOGIN_LONG.' 15';

		if (DB::query_result("SELECT COUNT(id) FROM {users_site} WHERE name='%s'", $name)) 
			return _LANG_LOGIN_EXISTS;
		return FALSE;
	}

	/**
	 * Проверяет e-mail на валидность
	 *
	 * @param string $mail электронный ящик
	 * @return string|boolean false
	*/
	public static function mail($mail)
	{
		if (! $mail)
			return _LANG_ENTER_EMAIL;

		if (substr($mail, 0, 1) == "'")
		{
			return _LANG_EMAIL_INCORRECT;
		}

		$name   = '[a-zA-Z0-9_\-\.\+\^!#\$%&*+\/\=\?\`\|\{\}~\']+';
		$domain = '(?:(?:[a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.?)+';
		$point  = '[0-9]{1,3}(\.[0-9]{1,3}){3}';
		$zone   = '[0-9a-fA-F]{1,4}(\:[0-9a-fA-F]{1,4}){7}';

		if (! preg_match("/^$name@($domain|(\[($point|$zone)\]))$/", $mail))
		{
			return _LANG_EMAIL_INCORRECT;
		}
		return FALSE;
	}

	/**
	 * Проверяет зарегистрирован ли пользователь с указанным электронным ящиком
	 *
	 * @param string $mail электронный ящик
	 * @return string|boolean false
	*/
	public static function mail_user($mail)
	{
		if (DB::query_result("SELECT COUNT(id) FROM {users_site} WHERE mail='%s'", $mail)) 
			return _LANG_MAIL_EXISTS;
		return FALSE;
	}

	/**
	 * Проверяет пароль на валидность
	 *
	 * @param string $password пароль
	 * @return string|boolean false
	*/
	public static function password($password)
	{
		if (!$password)
			return _LANG_ENTER_PASSWORD;
		return FALSE;
	}

	/**
	 * Проверяет текст на наличие длинных слов
	 *
	 * @param string $text текст
	 * @param integer $max_lenght максимальная длина слова
	 * @return string|boolean false
	*/
	public static function text($text, $max_lenght = 40)
	{
		$words = explode(" ", $text);
		for ($i = 0; $i< count($words); $i++)
		{
			if (utf::strlen($words[$i]) > $max_lenght)
			{
				return _LANG_ERR_MAX_LENGHT;
			}
		}
		return FALSE;
	}

	/**
	 * Проверяет URL на валидность
	 *
	 * @param string $url электорнный адрес
	 * @param boolean $absolute абсолютный адрес
	 * @return boolean
	*/
	public static function url($url, $absolute = FALSE)
	{
		$mask = '[a-z0-9\/:_\-_\.\?\$,;~=#&%\+]';
		if ($absolute)
		{
			return preg_match("/^(http|https|ftp):\/\/".$mask."+$/i", $url);
		}
		else
		{
			return preg_match("/^".$mask."+$/i", $url);
		}
	}
}