<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Comments_ajax
 *
 * Обработка запроса при добавления комментария
 */
class Comments_ajax extends Model
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array внутренние сообщения
	 */
	private $result;
        
        /**
	 * @var array поля формы
	 */
	private $param;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (empty($_POST['module']) || $_POST['module'] != 'comments')
		{
			return FALSE;
		}
		if (! empty($_POST["action"]) && $_POST["action"] == "list")
		{
			$this->diafan->_comments->get();
			return TRUE;
		}
		if (! $this->diafan->configmodules('security_user', 'comments') || $this->diafan->_user->id)
		{
			$id = $this->diafan->module
			      ? ($this->diafan->show ? $this->diafan->show : $this->diafan->cat)
			      : $this->diafan->cid;

			$module_name = $this->diafan->module
			               ? $this->diafan->module.(! $this->diafan->show ? "_category" : '')
			               : 'site';
                        
                        $this->param = $this->get_param($module_name);

			$this->check_captcha();
                        $this->empty_required_field();
			$this->valid_text();

			if ($this->send_errors())
				return TRUE;

			if ($this->error_insert($id, $module_name))
				return TRUE;

			if (! DB::query("INSERT INTO {comments} (created, module_name,element_id, user_id, text,act) VALUES ('%d', '%h','%d','%d','%h','%d')",
			               time(),
			               $module_name,
			               $id,
			               $this->diafan->_user->id,
			               $_POST["comment"],
			               $this->diafan->configmodules('security_moderation', 'comments') ? 0 : 1
			              )
			  )
			{
				$this->result["errors"][0] = $this->diafan->configmodules('error_add_message', 'comments');
				return $this->send_errors();
			}
                        
                        $save=DB::next_id("comments");
                        $this->insert_values($save);
                        
			//модерация сообщений
			if ($this->diafan->configmodules('security_moderation', 'comments'))
			{
				$mes = $this->diafan->configmodules('add_message', 'comments');
				$this->result["errors"][0] = $mes ? $mes : ' ';
				return $this->send_errors(TRUE);
			}

			if (! empty($_SERVER["HTTP_X_REQUESTED_WITH"]))
			{
				//выводит добавленное сообщение
				$row['date'] = $this->format_date(time(), "comments");
				$row["text"] = nl2br($this->diafan->get_param($_POST, "comment", '', 1));
				if ($this->diafan->configmodules('user_name', 'comments') && $this->diafan->_user->id)
				{
					$row["name"] = $this->get_author($this->diafan->_user->id);
				}
				
				$params = array();
				$q = DB::query("SELECT value,param_id FROM {comments_param_element} WHERE element_id=%d AND trash='0'", $save);
				while ($r = DB::fetch_array($q))
				    $params[$r['param_id']] = $r['value'];
				$row['param'] = $params;
				
				$result["rows"][] = $row;
				$result["new"]    = !empty($_POST["first"]) ? FALSE : TRUE;
				$result["param"] = $this->param;
				$this->result["add"] = $this->diafan->_tpl->get('get', 'comments', $result);
			}
			else
			{
				$mes = $this->diafan->configmodules('add_message', 'comments');
				$this->result["errors"][0] = $mes ? $mes : ' ';
			}
			$this->result["success"] = TRUE;

			return $this->send_errors();
		}
	}
        
        /**
	 * Получает массив полей формы комментария
	 * 
	 * @return array
	 */
	private function get_param($module_name)
	{

		$cache_meta = array(
				"name"     => "form_param",
				"language" => _LANG,
				"module_name"  => $module_name
			);
		if (! $rows_param = $this->diafan->_cache->get($cache_meta, "comments"))
		{
			$rows_param = array();
			$result = DB::query("SELECT id, [name], type, required, [text] FROM {comments_param} WHERE (module_name='%s' OR module_name='')"
			                    ." AND trash='0' ORDER BY sort ASC", $module_name);

			while ($row = DB::fetch_array($result))
			{
				if ($row["type"] == 'select' || $row["type"] == 'multiple')
				{
					$result_select = DB::query("SELECT [name], id FROM {comments_param_select} WHERE param_id=%d"
					                           ." ORDER BY sort ASC", $row["id"]);
					while ($row_select = DB::fetch_array($result_select))
					{
						$row["select_array"][] = $row_select;
					}
				}
				$rows_param[] = $row;
			}
			//сохранение кеша
			$this->diafan->_cache->save($rows_param, $cache_meta, "comments");
		}
		return $rows_param;
	}
        
        /**
	 * Проверяет на заполнение обязательных полей
	 * 
	 * @return boolean
	 */
	private function empty_required_field()
	{
		$message = array();
		foreach ($this->param as $row)
		{
			if ($row["required"] && empty($_POST['p'.$row["id"]]))
			{
				$this->result["errors"]["p".$row["id"]] = $this->diafan->configmodules('error_empty_fields', 'comments').' '.$row["name"];
			}
			if (empty($_POST["p".$row["id"]]) && $row["type"] != "checkbox")
				continue;

			if ($row["type"] == "email")
			{
				$this->valid_email($_POST["p".$row["id"]], "p".$row["id"]);
			}

			if ($row["type"] == "text" || $row["type"] == "textarea" || $row["type"] == "email")
			{
				$_POST["p".$row["id"]] = $this->diafan->get_param($_POST, "p".$row["id"], '', 1);
				$message[] = $row["name"].': '.$_POST["p".$row["id"]];
			}
			elseif ($row["type"] == "numtext")
			{
				$_POST["p".$row["id"]] = $this->diafan->get_param($_POST, "p".$row["id"], '', 2);
				$message[] = $row["name"].': '.$_POST["p".$row["id"]];
			}
			elseif ($row["type"] == "date")
			{
				$_POST["p".$row["id"]] = $this->diafan->unixdate($_POST["p".$row["id"]]);
				$message[] = $row["name"].': '.$this->format_date($_POST["p".$row["id"]]);
			}
			elseif ($row["type"] == "checkbox")
			{
				$value = ! empty($_POST["p".$row["id"]]) ? 1 : 0;
				$value_sel = DB::query_result("SELECT [name] FROM {comments_param_select} WHERE value=%d AND param_id=%d", $value, $row["id"]);
				if (! $value_sel && $value == 1)
				{
					$message[] = $row["name"];
				}
				else
				{
					$message[] = $row["name"].': '.$value_sel;
				}
			}
			elseif ($row["type"] == "select" && ! empty($row["select_array"]))
			{
				foreach ($row["select_array"] as $select)
				{
					if ($select["id"] == $_POST["p".$row["id"]])
					{
						$message[] = $row["name"].': '.$select["name"];
					}
				}
			}
			elseif ($row["type"] == "multiple" && ! empty($row["select_array"]) && ! empty($_POST["p".$row["id"]]) && is_array($_POST["p".$row["id"]]))
			{
				$vals = array();
				foreach ($row["select_array"] as $select)
				{
					if (in_array($select["id"], $_POST["p".$row["id"]]))
					{
						$vals[] = $select["name"];
					}
				}
				$message[] = $row["name"].': '.implode(", ", $vals);
			}
		}
		$this->result["message"] = implode('<br>', $message);
		return FALSE;
	}
        
        /**
	 * Проверка e-mail на валидность
	 * 
	 * @return boolean
	 */
	private function valid_email($email, $filed)
	{
		if ($email)
		{
			require_once(ABSOLUTE_PATH.'includes/validate.php');
			$mes = Validate::mail($email);
			if ($mes)
			{
				$this->result["errors"][$filed] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}


	/**
	 * Проверяет правильность каптчи
	 * 
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if ($this->diafan->configmodules('security_code', 'comments') == 1)
		{
			$mes = $this->diafan->_captcha->is_right('comments');
			$this->result["code"] = $this->diafan->_captcha->get('comments', '', TRUE);	
			if ($mes)
			{
				$this->result["errors"]["code"] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}
        
        /**
	 * Добавляет значение полей формы в базу данных
	 * 
	 * @return boolean true
	 */
	private function insert_values($save)
	{
		foreach ($this->param as $row)
		{
			if (empty($_POST["p".$row["id"]]))
				continue;

			if ($row["type"] == "multiple")
			{
				foreach ($_POST["p".$row["id"]] as $val)
				{
					DB::query("INSERT INTO {comments_param_element} (value, param_id, element_id) VALUES ('%h', '%d', '%d')",
					          $val, $row["id"], $save);
				}
			}
			else
			{
				DB::query("INSERT INTO {comments_param_element} (value, param_id, element_id) VALUES ('%h', '%d', '%d')",
				          $_POST["p".$row["id"]], $row["id"], $save);
			}
		}
		return TRUE;
	}

	/**
	 * Проверяет валидность комментария
	 * 
	 * @return boolean true
	 */
	private function valid_text()
	{
		if (! $_POST["comment"])
		{
			$mes = _LANG_EMPTY_COMMENT;
		}
		else
		{
			require_once(ABSOLUTE_PATH.'includes/validate.php');
			$mes = Validate::text($_POST["comment"]);
		}
		if ($mes)
		{
			$this->result["errors"][0] = $mes;
		}

		return TRUE;
	}

	/**
	 * Проверяет попытку отправить сообщение повторно
	 *
	 * @param integer $element_id номер элемента, к которому добавляется комментарий
	 * @param string $module_name модуль, к которому добавляется комментарий
	 * @return boolean
	 */
	private function error_insert($element_id, $module_name)
	{
		$mes = '';
		$num = DB::query_result("SELECT COUNT(id) FROM {comments} WHERE element_id='%d' AND (module_name='%h' OR module_name='') AND user_id='%d' AND text='%h' AND trash='0'",
		                        $element_id,
		                        $module_name,
		                        $this->diafan->_user->id,
		                        $_POST["comment"]
		                       );

		if ($num > 0)
		{
			$mes = $this->diafan->configmodules('error_insert_message', 'comments');
			$this->result["errors"][0] = $mes ? $mes : ' ';
		}
		return $this->send_errors();
	}

	/**
	 * Отправляет несколько ошибок
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if (! empty($this->result["errors"]) || ! empty($this->result["success"]))
		{
			if (! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if (! empty($this->result["errors"]))
				{
					foreach ($this->result["errors"] as $field => $mes)
					{
						if (empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->redirect($this->diafan->_route->current_link().($query ? '?error=comments'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}