<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Faq_ajax
 *
 * Обработка запроса при отправки сообщения из формы "Задайте вопрос"
 */
class Faq_ajax
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array результат выполнения запроса
	 */
	private $result;

	/**
	 * @var array страница, к которой прикреплена форма
	 */
	private $site_id;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (! empty($_POST['module']) && $_POST['module'] == 'faq' && ! empty($_POST["site_id"]) && ($this->diafan->configmodules('security', 'faq', $_POST["site_id"]) != 1 || $this->diafan->_user->id))
		{
			if (! $this->site_id = DB::query_result("SELECT id FROM {site} WHERE id=%d AND trash='0' AND addmodule='faq'", $_POST["site_id"]))
			{
				$this->result["errors"][0] = 'ERROR';
				$this->send_errors();
			}
			if (! empty($_POST["cat_id"]) && ! DB::query_result("SELECT id FROM {faq_category} WHERE id=%d AND trash='0' AND act='1'", $_POST["cat_id"]))
			{
				$this->result["errors"][0] = 'ERROR';
				$this->send_errors();
			}
			$this->check_captcha();
			$this->empty_field();
			$this->valid_email();

			if ($this->send_errors())
				return TRUE;

			if ($this->error_insert())
				return TRUE;

			if (! DB::query("INSERT INTO {faq} (created, mail, [anons], [name], cat_id, site_id) VALUES (%d, '%h', '%h', '%h', %d, %d)",
			               time(), $_POST["email"], $_POST["question"], $_POST["name"], $_POST["cat_id"], $this->site_id))
			{
				$mes = $this->diafan->configmodules('error_add_message', 'faq', $this->site_id);
				$this->result["errors"][0] = $mes ? $mes : ' ';
				return $this->send_errors();
			}
			$save = DB::next_id("faq");

			$err = '';
			$this->result["altnames"] = '';
			$result_upload = $this->diafan->_attachments->save($save, "faq", $this->site_id);
			if ($result_upload == 'success')
			{
				$this->result["altnames"] .= implode(' ', $_FILES['attachments']['name']);
			}
			elseif ($result_upload != 'empty')
			{
				DB::query("DELETE FROM {faq} WHERE id='%d'", $save);
				$this->diafan->_attachments->delete($save, 'faq');
				$this->result["errors"]["attachments"] = $result_upload;
				unset($this->result["altnames"]);
				return $this->send_errors();
			}

			$this->send_mail();
			unset($this->result["altnames"]);

			$mes = $this->diafan->configmodules('add_message', 'faq', $this->site_id);
			$this->result["errors"][0] = $mes ? $mes : ' ';
			$this->result["success"] = TRUE;
			return $this->send_errors();
		}
	}

	/**
	 * Проверяет правильность каптчи
	 * 
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if ($this->diafan->configmodules('security', 'faq', $this->site_id) == 2)
		{
			$mes = $this->diafan->_captcha->is_right('faq');
			$this->result["code"] = $this->diafan->_captcha->get('faq', '', TRUE);				
			if ($mes)
			{
				$this->result["errors"]["code"] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Проверяет e-mail на валидность
	 * 
	 * @return boolean true
	 */
	private function valid_email()
	{
		if ($_POST["email"])
		{
			require_once(ABSOLUTE_PATH.'includes/validate.php');
			$mes = Validate::mail($_POST["email"]);
			if ($mes)
			{
				$this->result["errors"]["email"] = $mes;
			}
		}
		return TRUE;
	}

	/**
	 * Проверяет на заполнение обязательных полей
	 * 
	 * @return boolean true
	 */
	private function empty_field()
	{
		if (empty($_POST['name']))
		{
			$this->result["errors"]["name"] = _LANG_ENTER_NAME;
		}
		if (empty($_POST['question']))
		{
			$this->result["errors"]["question"] = _LANG_ENTER_QUESTION;
		}
		return TRUE;
	}

	/**
	 * Проверяет на попытку отправить сообщение повторно
	 * 
	 * @return boolean
	 */
	private function error_insert()
	{
		$mes = '';
		$num = DB::query_result("SELECT COUNT(id) FROM {faq} where mail='%h' AND anons='%h'", $_POST["email"], $_POST["question"]);
		if ($num > 0)
		{ 
			$mes = $this->diafan->configmodules('error_insert_message', 'faq', $this->site_id);
			$this->result["errors"][0] = $mes ? $mes : ' ';
		}
		return $this->send_errors();
	}

	/**
	 * Уведомляет администратора по e-mail
	 * 
	 * @return boolean
	 */
	private function send_mail()
	{
		if (! $this->diafan->configmodules("sendmailadmin", 'faq', $this->site_id))
			return FALSE;
			
		$subject = str_replace(
			array('%title', '%url'),
			array(TITLE, BASE_URL),
			$this->diafan->configmodules("subject_admin", 'faq', $this->site_id)
		);
			
		$message = str_replace(
			array('%name', '%title', '%url', '%question', '%email', '%files'),
			array(
				$this->diafan->get_param($_POST, "name", '', 1),
				TITLE,
				BASE_URL,
				$this->diafan->get_param($_POST, "question", '', 1),
				$this->diafan->get_param($_POST, "email", '', 1),
				($this->result["altnames"] ? $this->result["altnames"] : '-')
			),
			$this->diafan->configmodules("message_admin", 'faq', $this->site_id)
		);

		$to   = $this->diafan->configmodules("emailconfadmin", 'faq', $this->site_id)
		        ? $this->diafan->configmodules("email_admin", 'faq', $this->site_id)
		        : EMAIL_CONFIG;
		$from = $this->diafan->configmodules("emailconf", 'faq', $this->site_id)
		        ? $this->diafan->configmodules("email", 'faq', $this->site_id)
		        : '';

		include_once(ABSOLUTE_PATH.'includes/mail.php');
		send_mail($to, $subject, $message, $from);

		return TRUE;
	}

	/**
	 * Отправляет ответ
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if (! empty($this->result["errors"]))
		{
			if (! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if (! empty($this->result["errors"]))
				{
					foreach ($this->result["errors"] as $field => $mes)
					{
						if (empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->redirect($this->diafan->_route->current_link().($query ? '?error=faq'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}