<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Faq_model
 *
 * Модель модуля "Вопрос-Ответ"
 */
class Faq_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Генерирует данные для списка всех вопросов без деления на категории
	 * 
	 * @return array
	 */
	public function list_()
	{
		if ($this->diafan->cat)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}

		$cache_meta = array(
				"name"     => "list",
				"language" => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"site_id"  => $this->diafan->cid
			);

		//кеширование
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			////navigation//
			$this->diafan->_paginator->page    = $this->diafan->page;
			$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
			$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
			$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(id) FROM {faq} WHERE [act]='1' AND trash='0' AND site_id=%d", $this->diafan->cid);
			$links = $this->diafan->_paginator->get();
			////navigation///

			$result = DB::query_range("SELECT id, created, [name], [anons], [text], timeedit, cat_id, site_id FROM {faq}"
			                          ." WHERE [act]='1' AND trash='0' AND site_id=%d"
			                          ." ORDER BY created DESC, id DESC",
			                          $this->diafan->cid, $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);

			$this->result["rows"] = $this->get_elements($result);

			$this->result["paginator"] = $links;
			
			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

				$this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';

				$this->result["rows"][$i]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'faq', _LANG);

				$this->result["rows"][$i]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'faq',  _LANG);

				$this->result["rows"][$i]["text"] = $this->diafan->_useradmin->get($row["text"], 'text', $row["id"], 'faq',  _LANG);

				$this->result["rows"][$i]["date"] = $this->diafan->_useradmin->get($row["date"], 'created', $row["id"], 'faq');
			}
		}
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;

	}

	/**
	 * Генерирует данные для первой страницы модуля "Вопрос-Ответ"
	 * 
	 * @return array
	 */
	public function first_page()
	{
		if ($this->diafan->page)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		//кеширование
		$cache_meta = array(
				"name"     => "first_page",
				"language" => _LANG,
				"site_id"  => $this->diafan->cid
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$result = DB::query("SELECT id, [name], [anons], timeedit, site_id FROM {faq_category}"
			                    ." WHERE [act]='1' AND parent_id=0 AND trash='0' AND site_id=%d"
			                    ." ORDER by sort ASC, id ASC", $this->diafan->cid);

			while ($row = DB::fetch_array($result))
			{
				$this->result["categories"][$row["id"]]["name"] = $row["name"];
				$this->result["categories"][$row["id"]]["anons"] = $row["anons"];

				if (empty($this->result["timeedit"]) || $row["timeedit"] > $this->result["timeedit"])
				{
					$this->result["timeedit"] = $row["timeedit"];
				}

				$this->result["categories"][$row["id"]]["children"] = array();
				$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {faq_category}"
				                             ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
				                             ." AND site_id=%d ORDER BY sort ASC, id ASC",
				                             $row["id"], $this->diafan->cid);

				while ($row_children = DB::fetch_array($result_children))
				{
					$child["link"]  = $this->diafan->_route->link($row_children["site_id"], 'faq', $row_children["id"]);
					$child["name"]  = $row_children["name"];
					$child["anons"] = $row_children["anons"];
					$this->result["categories"][$row["id"]]["children"][] = $child;
				}

				$children = array();
				if ($this->diafan->configmodules("childrens_elements"))
				{
					$children = $this->diafan->get_children($row["id"],"faq_category");
				}
				$children[] = $row["id"];

				$result_faq = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.[text], e.site_id, e.created FROM {faq} AS e"
				                              ." INNER JOIN {faq_category_rel} AS c ON e.id=c.element_id"
				                              ." WHERE c.cat_id IN (%s) AND e.[act]='1' AND e.trash='0'"
				                              ." GROUP BY e.id ORDER BY e.created DESC, e.id DESC",
				                              implode(',', $children),
				                              0, $this->diafan->configmodules("count_list")
				                             );
				$this->result["categories"][$row["id"]]["rows"] = $this->get_elements($result_faq);

				$this->result["categories"][$row["id"]]["link_all"] = $this->diafan->_route->link($row["site_id"], 'faq', $row["id"]);
			}

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		if (! empty($this->result["categories"]))
		{
			foreach ($this->result["categories"] as $cat => $rows)
			{
				if (! $rows)
					continue;

				$this->result["categories"][$cat]["name"] = $this->diafan->_useradmin->get($rows["name"], 'name', $cat, 'faq_category', _LANG);

				$this->result["categories"][$cat]["anons"] = $this->diafan->_useradmin->get($rows["anons"], 'anons', $cat, 'faq_category', _LANG);

				foreach ($rows["children"] as $k => $row)
				{
					$this->result["categories"][$cat]["children"][$k]["name"] =
						$this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'faq_category', _LANG);

					$this->result["categories"][$cat]["children"][$k]["anons"] =
						$this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'faq_category', _LANG);
				}

				foreach ($rows["rows"] as $k => $row)
				{
					$this->result["categories"][$cat]["rows"][$k]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

					$this->result["categories"][$cat]["rows"][$k]["show_rating"] =
						$this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';

					$this->result["categories"][$cat]["rows"][$k]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'faq', _LANG);

					$this->result["categories"][$cat]["rows"][$k]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'faq', _LANG);

					$this->result["categories"][$cat]["rows"][$k]["text"] = $this->diafan->_useradmin->get($row["text"], 'text', $row["id"], 'faq', _LANG);
		
					$this->result["categories"][$cat]["rows"][$k]["date"] = $this->diafan->_useradmin->get($row["date"], 'created', $row["id"], 'faq');
				}
			}
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для списка вопросов в категории
	 * 
	 * @return array
	 */
	public function list_category()
	{
		//кеширование
		$cache_meta = array(
				"name"     => "list",
				"cat_id"   => $this->diafan->cat,
				"language" => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"site_id"  => $this->diafan->cid
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$this->result["level"] = count($this->diafan->get_parents($this->diafan->cat, 'faq_category'));
			
			$this->result["path"] = $this->get_path('faq');

			$result = DB::query("SELECT id, [name], timeedit, [descr], [keywords], sort, parent_id, [title_meta] FROM {faq_category}"
			                    ." WHERE [act]='1' and id=%d AND trash='0' AND site_id=%d"
			                    ." ORDER BY sort ASC, id ASC",
			                    $this->diafan->cat, $this->diafan->cid);
				
			if (! $row = DB::fetch_array($result))
			{
				include_once(ABSOLUTE_PATH.'includes/404.php');
			}

			$this->result["id"]  = $row["id"];
			if ($this->diafan->configmodules("childrens_elements"))
			{
				$children = $this->diafan->get_children($this->diafan->cat, "faq_category");
			}
			$children[] = $this->diafan->cat;

			////navigation//
			$this->diafan->_paginator->page    = $this->diafan->page;
			$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
			$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
			$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {faq} as e"
			                                                  ." INNER JOIN {faq_category_rel} AS c ON e.id=c.element_id"
			                                                  ." AND e.id=c.element_id AND c.cat_id IN (%s)"
			                                                  ." WHERE e.[act]='1' AND e.trash='0'",
			                                                  implode(',', $children)
			                                                 );
			$links = $this->diafan->_paginator->get();
			////navigation///

			$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.[text], e.site_id, e.created FROM"
			                              ." {faq} AS e"
			                              ." INNER JOIN {faq_category_rel} AS c ON e.id=c.element_id AND c.cat_id IN (%s)"
			                              ." WHERE e.[act]='1' AND e.trash='0' GROUP BY e.id"
			                              ." ORDER BY e.created DESC, e.id DESC",
			                              implode(',', $children),
			                              $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr
			                             );

			$this->result["rows"] = $this->get_elements($result_img);

			$this->result["paginator"] = $links;

			$this->result["titlemodule"] = $row["name"];
			$this->result["edit_titlemodule"] = array("id" => $row["id"], "table" => "faq_category");
			$this->result["title_meta"]  = $row["title_meta"];
			$this->result["timeedit"]    = $row["timeedit"];
			$this->result["keywords"]    = $row["keywords"];
			$this->result["descr"]       = $row["descr"];

			$previous = DB::fetch_array(DB::query("SELECT [name], id FROM {faq_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                      ." AND (sort<%d OR sort=%d AND id<%d) AND parent_id=%d ORDER BY sort DESC, id DESC LIMIT 1",
			                                      $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if ($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				$this->result["previous"]["id"]   = $previous["id"];
				$this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "faq", $previous["id"]);
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id FROM {faq_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                  ." AND (sort>%d OR sort=%d AND id>%d) AND parent_id=%d ORDER BY sort ASC, id ASC LIMIT 1",
			                                  $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if ($next)
			{
				$this->result["next"]["text"] = $next["name"];
				$this->result["next"]["id"]   = $next["id"];
				$this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "faq", $next["id"]);
			}

			$this->result["children"] = array();
			$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {faq_category}"
			                             ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
			                             ." AND site_id=%d ORDER BY sort ASC, id ASC",
			                             $row["id"], $this->diafan->cid);

			while ($row_children = DB::fetch_array($result_children))
			{
				$child["link"]  = $this->diafan->_route->link($row_children["site_id"], 'faq', $row_children["id"]);
				$child["name"]  = $row_children["name"];
				$child["anons"] = $row_children["anons"];
				$this->result["children"][] = $child;
			}

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		$text = DB::query_result("SELECT [text] FROM {faq_category} WHERE id=%d", $this->diafan->cat);
		$this->result["text"] = $this->diafan->_useradmin->get($text, 'text', $this->diafan->cat, 'faq_category', _LANG);

		if ($this->diafan->configmodules("cat") && ! empty($this->diafan->cat) && $this->diafan->configmodules("comments_cat"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

				$this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';

				$this->result["rows"][$i]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'faq', _LANG);

				$this->result["rows"][$i]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'faq',  _LANG);

				$this->result["rows"][$i]["text"] = $this->diafan->_useradmin->get($row["text"], 'text', $row["id"], 'faq',  _LANG);

				$this->result["rows"][$i]["date"] = $this->diafan->_useradmin->get($row["date"], 'created', $row["id"], 'faq');
			}
			if (! empty($this->result["previous"]["text"]))
			{
				$this->result["previous"]["text"] =
					$this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'faq_category', _LANG);
			}
			if (! empty($this->result["next"]["text"]))
			{
				$this->result["next"]["text"] =
					$this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'faq_category', _LANG);
			}
			foreach ($this->result["children"] as $id => $row)
			{
				$this->result["children"][$id]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'faq_category', _LANG);
				$this->result["children"][$id]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'faq_category', _LANG);
			}
			foreach ($this->result["path"] as $k => $path)
			{
				if ($k == 0)
					continue;
				$this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'faq_category', _LANG);
			}
		}
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;
	}

	/**
	 * Генерирует данные для страницы вопроса
	 * 
	 * @return array|boolean false
	 */
	public function id()
	{
		//кеширование
		$cache_meta = array(
				"name"     => "show",
				"cat_id"   => $this->diafan->cat,
				"show"     => $this->diafan->show,
				"language" => _LANG,
				"site_id"  => $this->diafan->cid
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$row = DB::fetch_array(DB::query("SELECT id, [name], timeedit, created, cat_id, [keywords], [descr], [anons], site_id, [title_meta]"
			                                 ." FROM {faq}  WHERE [act]='1'"
			                                 ." AND id = %d AND trash='0' AND site_id=%d",
			                                 $this->diafan->show, $this->diafan->cid));
			if (empty($row))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}

			if (! $this->diafan->rewrite_module && ($this->diafan->configmodules("cat") && $this->diafan->cat != $row["cat_id"]
			   || ! $this->diafan->configmodules("cat") && $this->diafan->cat))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
				return FALSE;
			}
			$this->result["id"]     = $this->diafan->show;
			$this->diafan->cat      = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;
			$this->result["cat_id"] = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;
			$this->result["attachments"]   = $this->diafan->_attachments->get($row["id"], 'faq');
			$this->result["use_animation"] = $this->diafan->configmodules("use_animation");
			$this->result["name"]   = $row["name"];

			if ($this->result["cat_id"])
			{
				$this->result["allfaq"]["link"] = $this->diafan->_route->link($this->diafan->cid, "faq", $row["cat_id"]);
			}
			else
			{
				$this->result["allfaq"]["link"] = $this->diafan->_route->link($this->diafan->cid);
			}

			$previous = DB::fetch_array(DB::query("SELECT [anons], id, cat_id FROM {faq} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                      .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
			                                      ." AND (created<%d OR created=%d AND id<%d) ORDER BY created DESC, id DESC LIMIT 1",
			                                      $this->diafan->cid, $row["created"], $row["created"], $this->diafan->show));
			if ($previous)
			{
				$this->result["previous"]["text"] = $this->diafan->short_text($previous["anons"]);
				if (! $this->diafan->configmodules("cat"))
				{
					$previous["cat_id"] = 0;
				}
				$this->result["previous"]["id"]   = $previous["id"];
				$this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "faq", $previous["cat_id"], $previous["id"]);
			}
			$next = DB::fetch_array(DB::query("SELECT [anons], id, cat_id FROM {faq} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                  .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
			                                  ." AND (created>%d OR created=%d AND id>%d) ORDER BY created ASC, id ASC LIMIT 1",
			                                  $this->diafan->cid, $row["created"], $row["created"], $this->diafan->show));
			if ($next)
			{
				$this->result["next"]["text"] = $this->diafan->short_text($next["anons"]);
				if (! $this->diafan->configmodules("cat"))
				{
					$next["cat_id"] = 0;
				}
				$this->result["next"]["id"]   = $next["id"];
				$this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "faq", $next["cat_id"], $next["id"]);
			}

			$this->result["date"]           = $this->format_date($row['created']);

			//$this->result["titlemodule"]    = $row["name"];
			$this->result["title_meta"]     = $row["title_meta"];
			$this->result["timeedit"]       = $row["timeedit"];

			$this->result["keywords"]       = $row["keywords"];
			$this->result["descr"]          = $row["descr"];
			
			$this->result["path"]           = $this->get_path('faq');

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->diafan->cat = $this->result["cat_id"];

		$row = DB::fetch_array(DB::query("SELECT [text], [anons] FROM {faq} WHERE id=%d", $this->diafan->show));

		$this->result["name"]  = $this->diafan->_useradmin->get($this->result["name"], 'name', $this->result["id"], 'faq', _LANG);
		$this->result["date"] = $this->diafan->_useradmin->get($this->result["date"], 'created', $this->result["id"], 'faq');
		$this->result["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $this->result["id"], 'faq', _LANG);
		$this->result["text"]  = $this->diafan->_useradmin->get($row["text"], 'text', $this->result["id"], 'faq', _LANG);

		if (! empty($this->result["previous"]["text"]))
		{
			$this->result["previous"]["text"] =
				$this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'faq', _LANG);
		}
		if (! empty($this->result["next"]["text"]))
		{
			$this->result["next"]["text"] =
				$this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'faq', _LANG);
		}
		foreach ($this->result["path"] as $k => $path)
		{
			if ($k == 0)
				continue;
			$this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'faq_category', _LANG);
		}

		if ($this->diafan->configmodules("rating"))
		{
			$this->result["show_rating"] = $this->diafan->_rating->get();
			$this->result["add_rating"] = $this->diafan->_rating->add();
		}

		if ($this->diafan->configmodules("comments"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if ($this->diafan->configmodules("tags"))
		{
			$this->result["tags"] = $this->diafan->_tags->get();
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для формы добавления вопроса
	 *
	 * @param integer $site_id номер страницы
	 * @param integer $cat_id номер категории
	 * @param boolean $insert_form форма выводится с помощью шаблонного тэга
	 * @return array|boolean false
	 */
	public function form($site_id = 0, $cat_id = 0, $insert_form = FALSE)
	{
		if (! $insert_form)
		{
			$site_id = $this->diafan->cid;
			$cat_id = $this->diafan->cat;
		}
		else
		{
			if (! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='faq' LIMIT 1");
			}
			if (! $cat_id && $this->diafan->configmodules("cat", "faq", $site_id))
			{
				$cat_id = DB::query_result("SELECT id FROM {faq_category} WHERE site_id='%d' LIMIT 1", $site_id);
			}
		}
		if (! $site_id)
		{
			return FALSE;
		}
		$this->result["site_id"] = $site_id;

		if (! $this->diafan->configmodules("cat", "faq", $site_id))
		{
			$cat_id = 0;
		}

		if ($this->diafan->configmodules('security', 'faq', $site_id) != 1 || $this->diafan->_user->id)
		{
			$this->result["form"]["captcha"] = '';
			if ($this->diafan->configmodules('security', 'faq', $site_id) == 2)
			{
				$this->result["form"]["captcha"] = $this->diafan->_captcha->get("faq", $this->get_error("faq", "code"));
			}
			$this->result["form"]["site_id"] = $site_id;
			$this->result["form"]["cat_id"] = $cat_id;

			$this->result["form"]["attachments"] = FALSE;
			if ($this->diafan->configmodules('attachments', 'faq', $site_id) && $this->diafan->configmodules("max_count_attachments", 'faq', $site_id))
			{
				$this->result["form"]["attachments"]           = TRUE;
				$this->result["form"]["max_count_attachments"] = $this->diafan->configmodules("max_count_attachments", 'faq', $site_id);
				$this->result["form"]["error_attachments"] = $this->get_error("faq", 'attachments');
			}
			$this->result["form"]["error"]             = $this->get_error("faq");
			$this->result["form"]["error_name"]        = $this->get_error("faq", 'name');
			$this->result["form"]["error_question"]    = $this->get_error("faq", 'question');
			$this->result["form"]["error_email"]       = $this->get_error("faq", 'email');
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: вопросов и ответов
	 * 
	 * @param integer $count количество статей
	 * @param integer $cat_id категория
	 * @param boolean $often часто задаваемые вопросы
	 * @param string $sort сортировка date - по дате, rand - случайно
	 * @param integer $site_id страница сайта
	 * @return array
	 */
	public function show_block($count, $cat_id, $often, $sort, $site_id)
	{
		//кеширование
		$cache_meta = array(
				"name"     => "block",
				"cat_id"   => $cat_id,
				"count"    => $count,
				"language" => _LANG,
				"site_id"  => $site_id,
				"sort"     => $sort
			);

		if ($sort == "rand" || ! $this->result = $this->diafan->_cache->get($cache_meta, "faq"))
		{
			if (! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='faq' AND trash='0' AND act='1' AND block='0'");
			}
			if ($cat_id && $this->diafan->configmodules("cat", "faq"))
			{
				if ($this->diafan->configmodules("childrens_elements", "faq"))
				{ 
					$children = $this->diafan->get_children($cat_id, "faq_category");
				}
				$children[] = $cat_id;
			}

			if ($sort == "rand")
			{
				$max_count = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {faq} as e"
				                              .($cat_id && $this->diafan->configmodules("cat", "faq")
				                                ? " INNER JOIN {faq_category_rel} as c ON c.element_id=e.id"
				                                  ." AND c.cat_id IN (".implode(',', $children).")"
				                                : ''
				                               )
				                              ." WHERE e.[act]='1' AND e.trash='0'"
				                              .($often ? " AND often='1'" : "")
				                              .($site_id ? " AND e.site_id=".$site_id : '')
				                             );
				$rands = array();
				for ($i = 1; $i <= min($max_count, $count); $i++)
				{
					do
					{
						$rand = mt_rand(0, $max_count - 1);
					}
					while (in_array($rand, $rands));
					$rands[] = $rand;
				}
			}
			else
			{
				$rands[0] = 1;
			}
			$this->result["rows"] = array();

			foreach ($rands as $rand)
			{
				$result = DB::query_range("SELECT e.id, e.[name], e.[anons], e.[text], e.cat_id, e.timeedit, e.site_id, e.created"
				                          ." FROM {faq} AS e"
				                          .($cat_id && $this->diafan->configmodules("cat", "faq")
				                            ? " INNER JOIN {faq_category_rel} as c ON c.element_id=e.id"
				                              ." AND c.cat_id IN (".implode(',', $children).")"
				                            : ''
				                           )
				                          ." WHERE e.[act]='1' AND e.trash='0'"
				                          .($often ? " AND often='1'" : "")
				                          .($site_id ? " AND e.site_id=".$site_id : '')
				                          ." GROUP BY e.id"
				                          .($sort == "date" ? ' ORDER BY e.created DESC, e.id DESC' : ''),
				                          $sort == "rand" ? $rand : 0,
				                          $sort == "rand" ? 1     : $count);

				$this->result["rows"] = array_merge($this->result["rows"], $this->get_elements($result));
			}

			$this->result["name"] = $often ? _LANG_FREGUENTLY_ASKED_QUESTIONS : '';
			if (! empty($this->result["rows"]) && $cat_id && $this->diafan->configmodules("cat", "faq"))
			{
				$cat = DB::fetch_array(DB::query("SELECT [name], site_id, id FROM {faq_category} WHERE id=%d", $cat_id));
				
				$this->result["name"]     = $often ? $this->result["name"] : $cat["name"];
				$this->result["link_all"] = $this->diafan->_route->link($cat["site_id"], 'faq', $cat_id);
				$this->result["category"] = TRUE;
			}
			elseif (! empty($this->result["rows"]) && $site_id)
			{
				$this->result["name"]     = $often ? $this->result["name"] : DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $site_id);
				$this->result["link_all"] = $this->diafan->_route->link($site_id);
				$this->result["category"] = FALSE;
			}
			//сохранение кеша
			if ($sort != "rand")
			{
				$this->diafan->_cache->save($this->result, $cache_meta, "faq");
			}
		}

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'faq', _LANG);

				$this->result["rows"][$i]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'faq', _LANG);

				$this->result["rows"][$i]["text"] = $this->diafan->_useradmin->get($row["text"], 'text', $row["id"], 'faq', _LANG);

				$this->result["rows"][$i]["date"] = $this->diafan->_useradmin->get($row["date"], 'created', $row["id"], 'faq');
			}
		}

		return $this->result;
	}

	/**
	 * Форматирует данные о вопросам и ответах для списка
	 *
	 * @param resource $result результат выполнения SQL-запроса
	 * @return array
	 */
	public function get_elements($result)
	{
		if (empty($this->result["timeedit"]))
		{
			$this->result["timeedit"] = '';
		}
		$rows = array();
		while ($row = DB::fetch_array($result))
		{
			if (! $this->diafan->configmodules("cat", "faq", $row["site_id"]))
			{
				$row["cat_id"] = 0;
			}
			if ($row["timeedit"] < $this->result["timeedit"])
			{
				$this->result["timeedit"] = $row["timeedit"];
			}
			unset($row["timeedit"]);

			$row["link"] = $this->diafan->_route->link($row["site_id"], "faq", $row["cat_id"], $row["id"]);
			unset($row["cat_id"]);

			$row["anons"] = $this->diafan->short_text($row["anons"], 160);
			$row["text"]  = $this->diafan->short_text($row["text"], 160);
			$row["date"]  = $this->format_date($row['created'], "faq", $row["site_id"]);
			unset($row["created"]);

			$rows[] = $row;	
		}
		return $rows;
	}
}