<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Faq_view
 *
 * Шаблон модуля "Вопрос-Ответ"
 */
class Faq_view extends View
{
	/**
	 * Шаблон списка вопросов и ответов
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean
	 */
	public function list_($result)
	{
		echo '<div class="faq_list">';

		//описание текущей категории
		if (! empty($result["text"]))
		{
			echo '<div class="faq_cat_text">'.$result["text"].'</div>';
		}

		//подкатегории
		if (! empty($result["children"]))
		{
			foreach ($result["children"] as $child)
			{
				echo '<div class="faq_cat_link">';

				//название и ссылка подкатегории
				echo '<a href="'.BASE_PATH_HREF.$child["link"].'">'.$child["name"].'</a>';

				//краткое описание подкатегории
				if ($child["anons"])
				{
					echo '<div class="faq_cat_anons">'.$child["anons"].'</div>';
				}
				echo '</div>';
			}
		}
		
		//комментарии к категориям
		if (! empty($result["comments"]))
		{
			echo $result["comments"];
		}

		//вопросы
		if (! empty($result["rows"]))
		{
			foreach ($result["rows"] as $row)
			{
				echo '<div class="faq">';

				//дата вопроса
				if (! empty($row['date']))
				{
					echo '<div class="faq_date">'.$row["date"]."</div>";
				}

				//вопрос и ссылка на полную версию
				echo '<div class="faq_question">';
				echo '<a href="'.BASE_PATH_HREF.$row["link"].'">'.$row["anons"].'</a>';
					//рейтинг вопроса
					if (! empty($row["show_rating"])) echo ' ' .$row["show_rating"];
				echo '</div>';

				//ответ
				echo '<div class="faq_answer">'.$row["text"].'</div>';

				echo '</div>';
			}
			echo '<div class="clear"></div>';
		}

		//постраничная навигация
		if (! empty($result["paginator"]))
		{
			echo $result["paginator"];
		}

		//ссылки на предыдущую и последующую категории
		if (! empty($result["previous"]) || ! empty($result["next"]))
		{
			echo '<div class="previous_next_links">';
			if (! empty($result["previous"]))
			{
				echo '<div class="previous_link"><a href="'.BASE_PATH_HREF.$result["previous"]["link"].'">&larr; '.$result["previous"]["text"].'</a></div>';
			}
			if (! empty($result["next"]))
			{
				echo '<div class="next_link"><a href="'.BASE_PATH_HREF.$result["next"]["link"].'">'.$result["next"]["text"].' &rarr;</a></div>';
			}
			echo '</div>';
		}

		//форма добавления вопроса
		if (! empty($result["form"]))
		{
			self::form($result["form"]);
		}
		
		echo '</div>';
		
		return TRUE;
	}

	/**
	 * Шаблон первой страницы модуля "Впорос-Ответ"
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean
	 */
	public function first_page($result)
	{
		if (empty($result["categories"]))
			return FALSE;

		//категории
		foreach ($result["categories"] as $cat_id => $cat)
		{
			echo '<div class="faq_list">';

			//название категории
			echo '<h2>'.$cat["name"].'</h2>';

			//краткое описание категории
			if (! empty($cat["anons"]))
			{
				echo '<div class="faq_cat_anons">'.$cat["anons"].'</div>';
			}

			//подкатегории
			if (! empty($cat["children"]))
			{
				foreach ($cat["children"] as $child)
				{
					echo '<div class="faq_cat_link">';

					//название и ссылка подкатегории
					echo '<a href="'.BASE_PATH_HREF.$child["link"].'">'.$child["name"].'</a></div>';

					//краткое описание подкатегории
					if (! empty($child["anons"]))
					{
						echo '<div class="faq_cat_anons">'.$child["anons"].'</div>';
					}
					echo '</div>';
				}
			}
			
			//вопросы в категории
			if ($cat["rows"])
			{
				foreach ($cat["rows"] as $row)
				{
					echo '<div class="faq">';

					//дата вопроса
					if (! empty($row['date']))
					{
						echo '<div class="faq_date">'.$row["date"]."</div>";
					}

					//вопрос и ссылка на полную версию
					echo '<div class="faq_question">';
					echo '<a href="'.BASE_PATH_HREF.$row["link"].'">'.$row["anons"].'</a>';
						//рейтинг вопроса
						if (! empty($row["show_rating"])) echo $row["show_rating"];
					echo '</div>';

					//ответ
					echo '<div class="faq_answer">'.$row["text"].'</div>';

					echo '</div>';
				}
			}
			//ссылка на все вопросы в категории
			if ($cat["link_all"])
			{
				echo '<div class="show_all"><a href="'.BASE_PATH_HREF.$cat["link_all"].'">'
				.$this->_('SHOW_ALL_QUESTIONS_IN_CATEGORY').' '.$this->_('LQUOT').$cat["name"].$this->_('RQUOT')
				.'</a></div>';
			}
			echo '</div>';
		}

		//форма добавления вопроса
		if (! empty($result["form"]))
		{
			self::form($result["form"]);
		}

		return TRUE;
	}

	/**
	 * Шаблон страницы вопроса
	 * 
	 * @param array $row данные, сгенерированные в моделе
	 * @return boolean true
	 */
	public function id($row)
	{
		echo '<div class="faq_id">';

		//дата вопроса
		if (! empty($row["date"]))
		{
			echo '<div class="faq_date">';
			echo $row["date"];
				//рейтинг вопроса
				if (! empty($row["show_rating"])) echo $row["show_rating"];
			echo '</div>';
		}

		//вопрос
		echo '<div class="faq_question">'.$row["anons"]."</div>";

		//ответ
		echo '<div class="faq_answer">'.$row["text"]."</div>";

		//прикрепленные файлы
		if (! empty($row["attachments"]))
		{
			echo '<div class="faq_attachments">';
			foreach ($row["attachments"] as $att)
			{
				if ($att["is_image"])
				{
					if ($row["use_animation"])
					{
						$a_href  = '<a href="'.$att["link"].'" rel="prettyPhoto[gallery'.$att["element_id"].$att["module_name"].'_1]">';
						$a_href2 = '<a href="'.$att["link"].'" rel="prettyPhoto[gallery'.$att["element_id"].$att["module_name"].'_2]">';
					}
					else
					{
						$a_href .= '<a href="'.$att["link"].'" rel="big_image" width="'.$att["width"].'" height="'.$att["height"].'">';
						$a_href2 = $a_href;
					}
					echo
					'<p id="attachment'.$att["id"].'">'
						.$a_href.$att["name"].'</a>'
						.' ('.$att["size"].')'
						.' '.$a_href2.'<img src="'.$att["link_preview"].'"></a>'
					.'</p>';
				}
				else
				{
					echo '<p id="attachment'.$att["id"].'"><a href="'.$att["link"].'">'.$att["name"].'</a>  ('.$att["size"].')</p>';
				}
			}
			echo '</div>';
		}
		
		//форма оценки рейтинга вопроса
		if (! empty($row["add_rating"]))
		{
			echo $row["add_rating"];
		}
		
		//теги вопроса
		if (! empty($row["tags"]))
		{
			echo $row["tags"];
		}
		
		//комментарии к вопросу
		if (! empty($row["comments"]))
		{
			echo $row["comments"];
		}

		//ссылки на предыдущий и последующий вопрос
		if (! empty($row["previous"]) || ! empty($row["next"]))
		{
			echo '<div class="previous_next_links">';
			if (! empty($row["previous"]))
			{
				echo '<div class="previous_link"><a href="'.BASE_PATH_HREF.$row["previous"]["link"].'">&larr; '.$row["previous"]["text"].'</a></div>';
			}
			if (! empty($row["next"]))
			{
				echo '<div class="next_link"><a href="'.BASE_PATH_HREF.$row["next"]["link"].'">'.$row["next"]["text"].' &rarr;</a></div>';
			}
			echo '</div>';
		}

		//ссылки на все вопросы
		if (! empty($row["allfaq"]))
		{
			echo '<div class="show_all"><a href="'.BASE_PATH_HREF.$row["allfaq"]["link"].'">'.$this->_('BACK_TO_LIST').'</a></div>';
		}

		echo '</div>';

		//форма добавления вопроса
		if (! empty($row["form"]))
		{
			self::form($row["form"]);
		}

		return TRUE;		
	}

	/**
	 * Шаблонная функция: блок вопросов и ответов
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean
	 */
	public function show_block($result)
	{
		if (empty($result["rows"]))
		{
			return FALSE;
		}
		
		echo '<div class="faq_block">';

		//заголовок блока
		if (! empty($result["name"]))
		{
			echo '<h2>'.$result["name"].'</h2>';
		}

		//вопросы
		foreach ($result["rows"] as $row)
		{
			echo '<div class="faq">';

			//дата вопроса
			if (! empty($row["date"]))
			{
				echo '<div class="faq_date">'.$row["date"].'</div>';
			}
			//вопрос
			echo '<div class="faq_question"><a href="'.BASE_PATH_HREF.$row["link"].'">'.$row["anons"]."</a></div>";
	
			//ответ
			echo '<div class="faq_answer">'.$row["text"]."</div>";

			echo '</div>';
		}

		//ссылка на все вопросы
		if (! empty($result["link_all"]))
		{
			echo '<div class="show_all"><a href="'.BASE_PATH_HREF.$result["link_all"].'">';
			if ($result["category"])
			{
				echo $this->_('SHOW_ALL_QUESTIONS_IN_CATEGORY').' '.$this->_('LQUOT').$result["name"].$this->_('RQUOT');
			}
			else
			{
				echo $this->_('ALL_QUESTIONS');
			}
			echo '</a></div>';
		}
		
		echo '</div>';

		return TRUE;
	}
	
	/**
	 * Шаблон формы добавления вопроса
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean true
	 */
	public function form($result)
	{
		//заголовок блока
		echo '<h2>'.$this->_('ASK_YOUR_QUESTION').'</h2>';

		echo '<form method="POST" action="" enctype="multipart/form-data" class="ajax faq_form">
		<input type="hidden" name="module" value="faq">
		<input type="hidden" name="site_id" value="'.$result["site_id"].'">
		<input type="hidden" name="cat_id" value="'.$result["cat_id"].'">
		<input type="hidden" name="ajax" value="0">';

		//имя
		echo '<div class="infofield">'.$this->_('YOUR_NAME').'<font color="red">*</font>:</div>
		<input type = "text" maxLength="70" name="name" size="40" class="inptext" value=""><br>
		<div class="errors error_name"'.($result["error_name"] ? '>'.$result["error_name"] : ' style="display:none">').'</div>';

		//вопрос
		echo '<div class="infofield">'.$this->_('YOUR_QUESTION').'<font color="red">*</font>:</div>
		<textarea rows="10" name="question" cols="30" class="inptext"></textarea><br>
		<div class="errors error_question"'.($result["error_question"] ? '>'.$result["error_question"] : ' style="display:none">').'</div>';

		//e-mail
		echo '<div class="infofield">'.$this->_('YOUR_EMAIL').':</div>
		<input type = "text" maxLength="70" name="email" size="40" class="inptext" value=""><br>
		<div class="errors error_email"'.($result["error_email"] ? '>'.$result["error_email"] : ' style="display:none">').'</div>';

		//прикрепляемые файлы
		if ($result["attachments"])
		{
			echo '<div class="infofield">'.$this->_('ATTACHMENTS').':</div>
			<input type="file" name="attachments[]" class="inpfile" max="'.$result["max_count_attachments"].'">
			<div class="errors error_attachments"'.($result["error_attachments"] ? '>'.$result["error_attachments"] : ' style="display:none">').'</div><br>';
		}

		//защитный код
		echo $result["captcha"];

		//кнопка "Отправить"
		echo '<span class="button_wrap"><input type="submit" value="'.$this->_('SEND', FALSE).'" class="button" name="button"></span>';

		echo '<div class="errors error"'.($result["error"] ? '>'.$result["error"] : ' style="display:none">').'</div>';

		echo '</form>';

		return TRUE;
	}
}