<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Feedback_ajax
 *
 * Обработка запроса при отправке сообщения из формы обратной связи
 */
class Feedback_ajax extends Model
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array внутренние сообщения
	 */
	private $result;

	/**
	 * @var array страница, к которой прикреплена форма
	 */
	private $site_id;

	/**
	 * @var array поля формы
	 */
	private $param;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (! empty($_POST['module']) && $_POST['module'] == 'feedback' && ! empty($_POST["site_id"]) && ($this->diafan->configmodules('security', 'feedback', $_POST["site_id"]) != 1 || $this->diafan->_user->id))
		{
			if (! $this->site_id = DB::query_result("SELECT id FROM {site} WHERE id=%d AND trash='0' AND addmodule='feedback'", $_POST["site_id"]))
			{
				$this->result["errors"][0] = 'ERROR';
				$this->send_errors();
			}
			$this->param = $this->get_param();

			$this->check_captcha();
			$this->empty_required_field();

			if ($this->send_errors())
				return TRUE;

			if (! DB::query("INSERT INTO {feedback} (created, site_id, language) VALUES ('%d','%d','%d')", time(), $this->site_id, _LANG))
			{
				$mes = $this->diafan->configmodules('error_add_message', 'feedback', $this->site_id, _LANG);
				$this->result["errors"][0] = $mes ? $mes : ' ';
				return $this->send_errors();
			}
			$save = DB::next_id("feedback");

			$this->result["altnames"] = '';
			$result_upload = $this->diafan->_attachments->save($save, "feedback", $this->site_id);
			if ($result_upload == 'success')
			{
				$this->result["altnames"] .= implode(' ', $_FILES['attachments']['name']);
			}
			elseif ($result_upload != 'empty')
			{
				DB::query("DELETE FROM {feedback} WHERE id='%d'", $save);
				$this->diafan->_attachments->delete($save, 'feedback');
				$this->result["errors"]["attachments"] = $result_upload;
				unset($this->result["altnames"]);
				return $this->send_errors();
			}

			$this->insert_values($save);
			$this->send_mail();
			unset($this->result["altnames"]);

			$mes = $this->diafan->configmodules('add_message', 'feedback', $this->site_id, _LANG);
			$this->result["errors"][0] = $mes ? $mes : ' ';
			$this->result["success"] = TRUE;
			return $this->send_errors();
		}
		return FALSE;
	}

	/**
	 * Проверяет правильность каптчи
	 * 
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if ($this->diafan->configmodules('security', 'feedback', $this->site_id) == 2)
		{
			$mes = $this->diafan->_captcha->is_right('feedback');
			$this->result["code"] = $this->diafan->_captcha->get('feedback', '', TRUE);
			if ($mes)
			{
				$this->result["errors"]["code"] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Получает массив полей формы обратной связи
	 * 
	 * @return array
	 */
	private function get_param()
	{

		$cache_meta = array(
				"name"     => "form_param",
				"language" => _LANG,
				"site_id"  => $this->site_id
			);
		if (! $rows_param = $this->diafan->_cache->get($cache_meta, "feedback"))
		{
			$rows_param = array();
			$result = DB::query("SELECT id, [name], type, required, [text] FROM {feedback_param} WHERE site_id=%d"
			                    ." AND trash='0' ORDER BY sort ASC", $this->site_id);

			while ($row = DB::fetch_array($result))
			{
				if ($row["type"] == 'select' || $row["type"] == 'multiple')
				{
					$result_select = DB::query("SELECT [name], id FROM {feedback_param_select} WHERE param_id=%d"
					                           ." ORDER BY sort ASC", $row["id"]);
					while ($row_select = DB::fetch_array($result_select))
					{
						$row["select_array"][] = $row_select;
					}
				}
				$rows_param[] = $row;
			}
			//сохранение кеша
			$this->diafan->_cache->save($rows_param, $cache_meta, "feedback");
		}
		return $rows_param;
	}

	/**
	 * Проверяет на заполнение обязательных полей
	 * 
	 * @return boolean
	 */
	private function empty_required_field()
	{
		$message = array();
		foreach ($this->param as $row)
		{
			if ($row["required"] && empty($_POST['p'.$row["id"]]))
			{
				$this->result["errors"]["p".$row["id"]] = $this->diafan->configmodules('error_empty_fields', 'feedback', $this->site_id, _LANG).' '.$row["name"];
			}
			if (empty($_POST["p".$row["id"]]) && $row["type"] != "checkbox")
				continue;

			if ($row["type"] == "email")
			{
				$this->valid_email($_POST["p".$row["id"]], "p".$row["id"]);
			}

			if ($row["type"] == "text" || $row["type"] == "textarea" || $row["type"] == "email")
			{
				$_POST["p".$row["id"]] = $this->diafan->get_param($_POST, "p".$row["id"], '', 1);
				$message[] = $row["name"].': '.$_POST["p".$row["id"]];
			}
			elseif ($row["type"] == "numtext")
			{
				$_POST["p".$row["id"]] = $this->diafan->get_param($_POST, "p".$row["id"], '', 2);
				$message[] = $row["name"].': '.$_POST["p".$row["id"]];
			}
			elseif ($row["type"] == "date")
			{
				$_POST["p".$row["id"]] = $this->diafan->unixdate($_POST["p".$row["id"]]);
				$message[] = $row["name"].': '.$this->format_date($_POST["p".$row["id"]]);
			}
			elseif ($row["type"] == "checkbox")
			{
				$value = ! empty($_POST["p".$row["id"]]) ? 1 : 0;
				$value_sel = DB::query_result("SELECT [name] FROM {feedback_param_select} WHERE value=%d AND param_id=%d", $value, $row["id"]);
				if (! $value_sel && $value == 1)
				{
					$message[] = $row["name"];
				}
				else
				{
					$message[] = $row["name"].': '.$value_sel;
				}
			}
			elseif ($row["type"] == "select" && ! empty($row["select_array"]))
			{
				foreach ($row["select_array"] as $select)
				{
					if ($select["id"] == $_POST["p".$row["id"]])
					{
						$message[] = $row["name"].': '.$select["name"];
					}
				}
			}
			elseif ($row["type"] == "multiple" && ! empty($row["select_array"]) && ! empty($_POST["p".$row["id"]]) && is_array($_POST["p".$row["id"]]))
			{
				$vals = array();
				foreach ($row["select_array"] as $select)
				{
					if (in_array($select["id"], $_POST["p".$row["id"]]))
					{
						$vals[] = $select["name"];
					}
				}
				$message[] = $row["name"].': '.implode(", ", $vals);
			}
		}
		$this->result["message"] = implode('<br>', $message);
		return FALSE;
	}

	/**
	 * Проверка e-mail на валидность
	 * 
	 * @return boolean
	 */
	private function valid_email($email, $filed)
	{
		if ($email)
		{
			require_once(ABSOLUTE_PATH.'includes/validate.php');
			$mes = Validate::mail($email);
			if ($mes)
			{
				$this->result["errors"][$filed] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Добавляет значение полей формы в базу данных
	 * 
	 * @return boolean true
	 */
	private function insert_values($save)
	{
		foreach ($this->param as $row)
		{
			if (empty($_POST["p".$row["id"]]))
				continue;

			if ($row["type"] == "multiple")
			{
				foreach ($_POST["p".$row["id"]] as $val)
				{
					DB::query("INSERT INTO {feedback_param_element} (value, param_id, element_id) VALUES ('%h', '%d', '%d')",
					          $val, $row["id"], $save);
				}
			}
			else
			{
				DB::query("INSERT INTO {feedback_param_element} (value, param_id, element_id) VALUES ('%h', '%d', '%d')",
				          $_POST["p".$row["id"]], $row["id"], $save);
			}
		}
		return TRUE;
	}

	/**
	 * Уведомление администратора по e-mail
	 * 
	 * @return boolean
	 */
	private function send_mail()
	{
		if (! $this->diafan->configmodules("sendmailadmin", 'feedback', $this->site_id))
			return FALSE;

		$subject = str_replace(
			array('%title', '%url'),
			array(TITLE, BASE_URL),
			$this->diafan->configmodules("subject_admin", 'feedback', $this->site_id)
		);

		$message = str_replace(
			array('%title', '%url', '%message', '%files'),
			array(
				TITLE,
				BASE_URL,
				$this->result["message"],
				! empty($this->result["altnames"]) ? $this->result["altnames"] : '-'
			),
			$this->diafan->configmodules("message_admin", 'feedback', $this->site_id)
		);

		$to   = $this->diafan->configmodules("emailconfadmin", 'feedback', $this->site_id)
		        ? $this->diafan->configmodules("email_admin", 'feedback', $this->site_id)
		        : EMAIL_CONFIG;
		$from = $this->diafan->configmodules("emailconf", 'feedback', $this->site_id)
		        ? $this->diafan->configmodules("email", 'feedback', $this->site_id)
		        : '';

		//$this->result["errors"]["attachments"] = $subject.'<br><br>'.$message;
		include_once(ABSOLUTE_PATH.'includes/mail.php');
		send_mail($to, $subject, $message, $from);

		return TRUE;
	}

	/**
	 * Отправляет несколько ошибок
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if (! empty($this->result["errors"]))
		{
			if (! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if (! empty($this->result["errors"]))
				{
					foreach ($this->result["errors"] as $field => $mes)
					{
						if (empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->redirect($this->diafan->_route->current_link().($query ? '?error=feedback'.$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}