<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Images_inc
 * Работа с изображениями
 */
class Images_inc
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Получает изображения, прикрепленные к элементу модуля
	 *
	 * @param integer $id номер элемента, к которому прикреплены изображения
	 * @param string $alt альтернативный текст получаемых изображений
	 * @param string $module_name название модуля, по умолчанию модуль, прикрепленный к текущей странице
	 * @param integer $count количество изображений
	 * @param string|integer $common_link ссылка, на которую ведет изображение | 1 — сылка ведет на увеличенную копию изображений
	 * @param boolean $is_category изображения прикреплены к категории
	 * @return array
	 */
	public function get($id, $alt, $module_name, $count = 0, $common_link = 1, $is_category = FALSE)
	{
		global $diafan;
		$images = array();
		$result = DB::query("SELECT id, name, size, [alt], [title] FROM {images} WHERE module_name='%s' AND element_id='%d' AND trash='0' ORDER BY sort ASC"
		                    .($count ? " LIMIT ".$count : ''), $module_name.($is_category ? 'cat' : ''), $id);
		
		while ($row = DB::fetch_array($result))
		{
			if(!file_exists(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"])) continue;
			if ($common_link == 1)
			{
				if (! $diafan->configmodules('use_animation', $module_name))
				{
					list($img["link_width"], $img["link_height"]) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"]);
					$img["link"] = USERFILES.'/'.$module_name.'/'.$row["name"];
					$img["type"] = "big_image";
				}
				else
				{
					$img["link"] = USERFILES.'/'.$module_name.'/'.$row["name"];
					$img["type"] = "animation";
				}
			}
			else
			{
				$img["type"] = "link";
				$img["link"] = $common_link;
			}
			list($img["width"], $img["height"]) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/medium/'.$row["name"]);
			$img["alt"] = $row["alt"] ? $row["alt"] : $alt;
			$img["title"] = $row["title"] ? $row["title"] : $alt;
			$img["src"] = BASE_PATH.USERFILES.'/'.$module_name.'/medium/'.$row["name"];
			$img["size"] = $diafan->convert($row["size"]);
			$images[] = $img;
			
			if ($count == 1)
			{
				return $images;
			}
		}
		return $images;
	}

	/**
	 * Получает большие копии изображений, прикрепленных к элементу модуля
	 *
	 * @param integer $id номер элемента, к которому прикреплены изображения
	 * @param string $alt альтернативный текст получаемых изображений
	 * @param string $module_name название модуля, по умолчанию модуль, прикрепленный к текущей странице
	 * @param integer $count количество изображений
	 * @param boolean $is_category изображения прикреплены к категории
	 * @return array
	 */
	public function get_big($id, $alt, $module_name, $count = 0, $is_category = FALSE)
	{
		global $diafan;
		$images = array();
		$result = DB::query("SELECT id, name, size, [alt], [title] FROM {images} WHERE module_name='%s' AND element_id='%d' AND trash='0' ORDER BY sort ASC"
		                    .($count ? " LIMIT ".$count : ''), $module_name.($is_category ? 'cat' : ''), $id);

		while ($row = DB::fetch_array($result))
		{
			list($img["width"], $img["height"]) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"]);
			$img["type"]  = "image";
			$img["alt"]   = $row["alt"] ? $row["alt"] : $alt;
			$img["title"] = $row["title"] ? $row["title"] : $alt;
			$img["src"]   = BASE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"];

			if ($count == 1)
			{
				return $img;
			}
			$images[] = $img;
		}
		return $images;
	}

	/*
	 * Удаляет прикрепленные изображения
	 *
	 * @param integer $element_id номер элемента, к которому прикреплены изображения
	 * @param string $module_name название модуля
	 * @param integer|boolean false $image_id номер изображения
	 * @return boolean true
	*/
	public function delete($element_id, $module_name, $image_id = FALSE)
	{
		if ($image_id)
		{
			$result = DB::query("SELECT * FROM {images} WHERE id=%d AND element_id=%d AND module_name='%h'", $image_id, $element_id, $module_name);
		}
		else
		{
			$result = DB::query("SELECT * FROM {images} WHERE element_id=%d AND module_name='%h'", $element_id, $module_name);
		}
		while ($row = DB::fetch_array($result))
		{
			DB::query("DELETE FROM {images} WHERE id='%d'", $row["id"]);
			@unlink(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$row["name"]);
			@unlink(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/small/'.$row["name"]);
			@unlink(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/medium/'.$row["name"]);
		}
		return TRUE;
	}
}