<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Menu_model
 *
 * Модель модуля "Меню"
 */
class Menu_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Генерирует данные для шаблонной функции: блок меню
	 *
	 * @param integer $id номер меню
	 * @return array
	 */
	public function show_block($id)
	{
		//кеширование
		$cache_meta = array(
				"name"     => "block",
				"id"       => $id,
				"language" => _LANG
			);

		if (! $this->result = $this->diafan->_cache->get($cache_meta, "menu"))
		{
			if (! $id)
			{
				$id = 1;
			}
			$row_menu = DB::fetch_array(DB::query("SELECT [name], show_all_level, hide_parent_link, show_title FROM {menu_category} WHERE id=%d AND [act]='1' AND trash='0'", $id));
			if (! $row_menu)
			{
				return FALSE;
			}
			if ($row_menu["show_title"])
			{
				$this->result["name"] = $row_menu["name"];
			}
			$this->result["show_all_level"]   = $row_menu["show_all_level"];
			$this->result["hide_parent_link"] = $row_menu["hide_parent_link"];
			$this->result["rows"] = array();

			$result = DB::query("SELECT id, [name], module_name, site_id, module_cat_id, element_id, parent_id, othurl FROM {menu}"
			                    ." WHERE cat_id=%d AND [act]='1' AND trash='0' ORDER BY sort ASC", $id);
			while ($row = DB::fetch_array($result))
			{
				if ($row["module_name"] == "site")
				{
					$row["link"] = $this->diafan->_route->link($row["site_id"]);
				}
				else
				{
					if ($row["module_cat_id"] && ! $this->diafan->configmodules("cat", $row["module_name"], $row["site_id"]))
					{
						continue;
					}
					$row["link"] = $this->diafan->_route->link($row["site_id"], $row["module_name"], $row["module_cat_id"], $row["element_id"]);
				}
				$row["active"] = FALSE;
				$row["active_child"] = FALSE;
				$this->result["rows"][$row["parent_id"]][] = $row;
			}
			$this->diafan->_cache->save($this->result, $cache_meta, "menu");
		}
		$this->result["parent_id"] = 0;
		$this->result["level"] = 1;

		$this->menu_active_chain();

		return $this->result;
	}

	/**
	 * Выделяет пункты меню активной цепи (текущая страница и ее родители)
	 *
	 * @return boolean true
	 */
	private function menu_active_chain()
	{
		$current_link = $this->diafan->_route->current_link(array('page'));
		$current_link_find = FALSE;
		$parents = array();
		foreach ($this->result["rows"] as $parent_id => $rows)
		{
			foreach ($rows as $i => $row)
			{
				$this->result["rows"][$parent_id][$i]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'menu', _LANG);
				if (empty($row["othurl"]) && $row["link"] == $current_link)
				{
					$this->result["rows"][$parent_id][$i]["active"] = TRUE;
					$current_link_find = $row["id"];
				}
				$parents[$row["id"]] = $row["parent_id"];
			}
		}
		if (! $current_link_find)
		{
			foreach ($this->result["rows"] as $parent_id => $rows)
			{
				foreach ($rows as $i => $row)
				{
					if ($row["element_id"] && $row["site_id"] == $this->diafan->cid && $row["element_id"] == $this->diafan->show)
					{
						$this->result["rows"][$parent_id][$i]["active_child"] = TRUE;
						$current_link_find = $row["id"];
						continue;
					}
				}
				if ($current_link_find)
					continue;
			}
		}
		if (! $current_link_find)
		{
			foreach ($this->result["rows"] as $parent_id => $rows)
			{
				foreach ($rows as $i => $row)
				{
					if (! $row["element_id"] && $row["module_cat_id"] && $row["site_id"] == $this->diafan->cid && $row["module_cat_id"] == $this->diafan->cat)
					{
						$this->result["rows"][$parent_id][$i]["active_child"] = TRUE;
						$current_link_find = $row["id"];
						continue;
					}
				}
				if ($current_link_find)
					continue;
			}
		}
		if (! $current_link_find)
		{
			foreach ($this->result["rows"] as $parent_id => $rows)
			{
				foreach ($rows as $i => $row)
				{
					if ($row["module_name"] == 'site' && $row["site_id"] == $this->diafan->cid)
					{
						$this->result["rows"][$parent_id][$i]["active_child"] = TRUE;
						$current_link_find = $row["id"];
						continue;
					}
				}
				if ($current_link_find)
					continue;
			}
		}
		if ($current_link_find)
		{
			$parent_id = $parents[$current_link_find];
			while ($parent_id > 0)
			{
				$id = $parent_id;
				$parent_id = $parents[$parent_id];
				foreach ($this->result["rows"][$parent_id] as $i => $row)
				{
					if ($row["id"] == $id)
					{
						$this->result["rows"][$parent_id][$i]["active_child"] = TRUE;
						continue;
					}
				}
			}
		}
		return TRUE;
	}
}
