<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
    include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Menu_view
 *
 * Шаблон модуля "Меню"
 */
class Menu_view extends View
{

    /**
     * Шаблонная функция: вывод меню
     * Выполняется в том случае, если передан параметр templates=default при вызове тега 
     * <insert name="show_block" module="menu" id="1" template="default"> 
     * 
     * @param array $result данные, сгенерированные в моделе
     * @return boolean
     */
    public function show_block($result)
    {
	if (empty($result["rows"]))
	{
	    return FALSE;
	}
	if (!empty($result["name"]))
	{
	    echo '<h2>'.$result["name"].'</h2>';
	}

	echo '<ul class="left_menu_level_1">';
	$this->get('show_level', 'menu', $result);  //ниже функция show_level(), которая выводит в цикле пункты первого уровня вложенности
	echo '</ul>';

	return TRUE;
    }

    /**
     * Шаблонная функция: вывод меню
     * Полный аналог функции show_block, но с другим оформлением. 
     * Нужен, если необходимо оформить другое меню на сайте
     * Вызывается с параметром templates=topmenu при вызове тега. 
     * <insert name="show_block" module="menu" id="1" template="topmenu"> 
     * Параметр должен быть приклеен к имени функции в конце
     * 
     * @param array $result данные, сгенерированные в моделе
     * @return boolean
     */
    public function show_block_topmenu($result)
    {
	if (empty($result["rows"]))
	{
	    return FALSE;
	}
	if (!empty($result["name"]))
	{
	    echo '<h2>'.$result["name"].'</h2>';
	}

	echo '<ul class="sf-menu">';
	$this->get('show_level_topmenu', 'menu', $result); //ниже функция show_level_topmenu(), которая выводит в цикле пункты первого уровня вложенности
	echo '</ul>';

	return TRUE;
    }

    /**
     * Шаблон вывода первого уровня меню, вызывается из функции show_block в начале файла
     * 
     * @param array $result данные, сгенерированные в моделе
     * @return boolean
     */
    public function show_level($result)
    {
	if (empty($result["rows"][$result["parent_id"]]))
	{
	    return TRUE;
	}

	// начало уровня меню
	foreach ($result["rows"][$result["parent_id"]] as $row)
	{
	    if (!$result["rows"][$result["parent_id"]][0] != $row)
	    {
		// разделитель пунктов меню
	    }

	    if ($row["active"])
	    {
		// начало пункта меню для текущей страницы
		echo '<li class="active">';
	    }
	    elseif ($row["active_child"])
	    {
		// начало пункта меню для активного дочернего пункта
		echo '<li class="active_child">';
	    }
	    else
	    {
		// начало любого другого пункта меню
		echo '<li>';
	    }

	    if (
	    // на текущей странице нет ссылки
		    !$row["active"]

		    // влючен пункт "Не отображать ссылку на элемент, если он имеет дочерние пункты"
		    && (!$result["hide_parent_link"] || empty($result["rows"][$row["id"]]))
	    )
	    {
		if ($row["othurl"])
		{
		    echo '<a href="'.$row["othurl"].'" target="_blank">';
		}
		else
		{
		    echo '<a href="'.BASE_PATH_HREF.$row["link"].'">';
		}
	    }
	    // название пункта меню
	    echo $row["name"];
	    if (
	    // на текущей странице нет ссылки
		    !$row["active"]

		    // влючен пункт "Не отображать ссылку на элемент, если он имеет дочерние пункты"
		    && (!$result["hide_parent_link"] || empty($result["rows"][$row["id"]]))
	    )
	    {
		echo '</a>';
	    }



	    if ($row["active"])
	    {
		// окончание пункта меню - текущей страницы
		echo '</li>';
	    }
	    elseif ($row["active_child"])
	    {
		// окончание пункта меню для активного дочернего пункта
		echo '</li>';
	    }
	    else
	    {
		// окончание любого другого пункта меню
		echo '</li>';
	    }

	    if ($result["show_all_level"] || $row["active_child"] || $row["active"])
	    {
		// вывод вложенного уровня меню
		$menu_data = $result;
		$menu_data["parent_id"] = $row["id"];
		$menu_data["level"]++;

		if (empty($result['attributes']['count_level']) || $result['attributes']['count_level'] >= $menu_data["level"])
		    $this->get('show_level_2', 'menu', $menu_data);  //ниже функция, которая выводит в цикле пункты второго уровня вложенности.
	    }
	}
	// окончание уровня меню

	return TRUE;
    }

    /**
     * Шаблон вывода второго и последующих уровней меню
     * 
     * @param array $result данные, сгенерированные в моделе
     * @return boolean
     */
    public function show_level_2($result)
    {
	if (empty($result["rows"][$result["parent_id"]]))
	{
	    return TRUE;
	}

	// начало уровня меню
	echo '<ul class="left_menu_level_'.$result["level"].'">';
	foreach ($result["rows"][$result["parent_id"]] as $row)
	{
	    if (!$result["rows"][$result["parent_id"]][0] != $row)
	    {
		// разделитель пунктов меню
	    }

	    if ($row["active"])
	    {
		// начало пункта меню для текущей страницы
		echo '<li class="menu_item_'.$result["level"].'">';
	    }
	    elseif ($row["active_child"])
	    {
		// начало пункта меню для активного дочернего пункта
		echo '<li class="menu_item_'.$result["level"].' menu_active">';
	    }
	    else
	    {
		// начало любого другого пункта меню
		echo '<li class="menu_item_'.$result["level"].' menu_active_child">';
	    }

	    if (
	    // на текущей странице нет ссылки
		    !$row["active"]

		    // влючен пункт "Не отображать ссылку на элемент, если он имеет дочерние пункты"
		    && (!$result["hide_parent_link"] || empty($result["rows"][$row["id"]]))
	    )
	    {
		if ($row["othurl"])
		{
		    echo '<a href="'.$row["othurl"].'" target="_blank">';
		}
		else
		{
		    echo '<a href="'.BASE_PATH_HREF.$row["link"].'">';
		}
	    }
	    // название пункта меню
	    echo $row["name"];
	    if (
	    // на текущей странице нет ссылки
		    !$row["active"]

		    // влючен пункт "Не отображать ссылку на элемент, если он имеет дочерние пункты"
		    && (!$result["hide_parent_link"] || empty($result["rows"][$row["id"]]))
	    )
	    {
		echo '</a>';
	    }

	    if ($result["show_all_level"] || $row["active_child"] || $row["active"])
	    {
		// вывод вложенного уровня меню
		$menu_data = $result;
		$menu_data["parent_id"] = $row["id"];
		$menu_data["level"]++;

		if (empty($result['attributes']['count_level']) || $result['attributes']['count_level'] >= $menu_data["level"])
		    $this->get('show_level_2', 'menu', $menu_data);  //вызывает сама себя, для вывод последующих уровней вложенности с классом class="menu_item_2 и т.д.
	    }

	    if ($row["active"])
	    {
		// окончание пункта меню - текущей страницы
		echo '</li>';
	    }
	    elseif ($row["active_child"])
	    {
		// окончание пункта меню для активного дочернего пункта
		echo '</li>';
	    }
	    else
	    {
		// окончание любого другого пункта меню
		echo '</li>';
	    }
	}
	// окончание уровня меню
	echo '</ul>';

	return TRUE;
    }

    /**
     * Шаблон вывода первого уровня меню, вызывается из функции show_block в начале файла
     * 
     * @param array $result данные, сгенерированные в моделе
     * @return boolean
     */
    public function show_level_topmenu($result)
    {
	if (empty($result["rows"][$result["parent_id"]]))
	{
	    return TRUE;
	}
	$k = 0;
	echo '<li class="top_menu_separator_1"></li>';
	// начало уровня меню
	foreach ($result["rows"][$result["parent_id"]] as $row)
	{
	    if ($k)
	    {
		echo '<li class="top_menu_separator_1"></li>';
	    }
	    $k = 1;

	    if ($row["active"])
	    {
		// начало пункта меню для текущей страницы
		echo '<li>';
	    }
	    elseif ($row["active_child"])
	    {
		// начало пункта меню для активного дочернего пункта
		echo '<li>';
	    }
	    else
	    {
		// начало любого другого пункта меню
		echo '<li>';
	    }


	    if ($row["othurl"])
	    {
		echo '<a href="'.$row["othurl"].'" target="_blank"    '
		.(!empty($row["active"]) || !empty($row["active_child"]) ? 'class=" active"' : '')
		.'>';
	    }
	    else
	    {
		echo '<a href="'.BASE_PATH_HREF.$row["link"].'"   '
		.(!empty($row["active"]) || !empty($row["active_child"]) ? 'class=" active"' : '')
		.'>';
	    }



	    // название пункта меню
	    echo $row["name"];


	    echo '</a>';
	    if ($result["show_all_level"] || $row["active_child"] || $row["active"])
	    {
		// вывод вложенного уровня меню
		$menu_data = $result;
		$menu_data["parent_id"] = $row["id"];
		$menu_data["level"]++;

		$this->get('show_level_topmenu_2', 'menu', $menu_data);
	    }

	    if ($row["active"])
	    {
		// окончание пункта меню - текущей страницы
		echo '</li>';
	    }
	    elseif ($row["active_child"])
	    {
		// окончание пункта меню для активного дочернего пункта
		echo '</li>';
	    }
	    else
	    {
		// окончание любого другого пункта меню
		echo '</li>';
	    }
	}
	// окончание уровня меню

	return TRUE;
    }

    /**
     * Шаблон вывода второго и последующих уровней меню
     * 
     * @param array $result данные, сгенерированные в моделе
     * @return boolean
     */
    public function show_level_topmenu_2($result)
    {
	if (empty($result["rows"][$result["parent_id"]]))
	{
	    return TRUE;
	}
	// начало уровня меню
	echo '<ul>';
	$k = 0;
	foreach ($result["rows"][$result["parent_id"]] as $row)
	{
	    if (!$result["rows"][$result["parent_id"]][0] != $row && $k)
	    {
		// разделитель пунктов меню
		echo '<li class="top_menu_separator_2">';
	    }
	    $k=1;

	    if ($row["active"])
	    {
		// начало пункта меню для текущей страницы
		echo '<li>';
	    }
	    elseif ($row["active_child"])
	    {
		// начало пункта меню для активного дочернего пункта
		echo '<li>';
	    }
	    else
	    {
		// начало любого другого пункта меню
		echo '<li>';
	    }

	    if ($row["othurl"])
	    {
		echo '<a href="'.$row["othurl"].'" target="_blank">';
	    }
	    else
	    {
		echo '<a href="'.BASE_PATH_HREF.$row["link"].'">';
	    }

	    // название пункта меню
	    echo $row["name"];

	    echo '</a>';

	    if ($result["show_all_level"] || $row["active_child"] || $row["active"])
	    {
		// вывод вложенного уровня меню
		$menu_data = $result;
		$menu_data["parent_id"] = $row["id"];
		$menu_data["level"]++;

		$this->get('show_level_topmenu_2', 'menu', $menu_data);
	    }
	    if ($row["active"])
	    {
		// окончание пункта меню - текущей страницы
		echo '</li>';
	    }
	    elseif ($row["active_child"])
	    {
		// окончание пункта меню для активного дочернего пункта
		echo '</li>';
	    }
	    else
	    {
		// окончание любого другого пункта меню
		echo '</li>';
	    }
	}
	// окончание уровня меню
	echo '</ul>';

	return TRUE;
    }

    //********************************************************************************************

    /**
     * Шаблонная функция: простой вывод меню, если не передан параметр templates,
     * но есть атрибуты tag_1, tag_2 и т.д.
     * 
     * @param array $result данные, сгенерированные в моделе
     * @param integer $parent_id родитель для текущего уровня
     * @param integer $level текущий уровень меню
     * @return boolean
     */
    public function show_menu($result, $parent_id = 0, $level = 1)
    {
	$result["attributes"]["tag_level_start_".$level] = isset($result["attributes"]["tag_level_start_".$level]) ? $result["attributes"]["tag_level_start_".$level] : $result["attributes"]["tag_level_start_".($level - 1)];
	$result["attributes"]["tag_level_end_".$level] = isset($result["attributes"]["tag_level_end_".$level]) ? $result["attributes"]["tag_level_end_".$level] : $result["attributes"]["tag_level_end_".($level - 1)];

	$result["attributes"]["tag_active_child_start_".$level] = isset($result["attributes"]["tag_active_child_start_".$level]) ? $result["attributes"]["tag_active_child_start_".$level] : $result["attributes"]["tag_active_child_start_".($level - 1)];
	$result["attributes"]["tag_active_child_end_".$level] = isset($result["attributes"]["tag_active_child_end_".$level]) ? $result["attributes"]["tag_active_child_end_".$level] : $result["attributes"]["tag_active_child_end_".($level - 1)];

	$result["attributes"]["tag_start_".$level] = isset($result["attributes"]["tag_start_".$level]) ? $result["attributes"]["tag_start_".$level] : $result["attributes"]["tag_start_".($level - 1)];
	$result["attributes"]["tag_end_".$level] = isset($result["attributes"]["tag_end_".$level]) ? $result["attributes"]["tag_end_".$level] : $result["attributes"]["tag_end_".($level - 1)];

	$result["attributes"]["tag_active_start_".$level] = isset($result["attributes"]["tag_active_start_".$level]) ? $result["attributes"]["tag_active_start_".$level] : $result["attributes"]["tag_active_start_".($level - 1)];
	$result["attributes"]["tag_active_end_".$level] = isset($result["attributes"]["tag_active_end_".$level]) ? $result["attributes"]["tag_active_end_".$level] : $result["attributes"]["tag_active_end_".($level - 1)];

	$result["attributes"]["separator_".$level] = isset($result["attributes"]["separator_".$level]) ? $result["attributes"]["separator_".$level] : $result["attributes"]["separator_".($level - 1)];
	$result["attributes"]["site_id"] = isset($result["attributes"]["site_id"]) ? $result["attributes"]["site_id"] : 0;

	$k = 0;
	if (!empty($result["rows"][$parent_id]))
	{
	    echo $result["attributes"]["tag_level_start_".$level];
	    foreach ($result["rows"][$parent_id] as $row)
	    {
		echo ($k ? $result["attributes"]["separator_".$level] : '');
		if ($row["active"] && $result["attributes"]["tag_active_start_".$level])
		{
		    echo str_replace(array('Increment', 'Level'), array($k, $level), $result["attributes"]["tag_active_start_".$level]);
		}
		elseif ($row["active_child"] && $result["attributes"]["tag_active_child_start_".$level])
		{
		    echo str_replace(array('Increment', 'Level'), array($k, $level), $result["attributes"]["tag_active_child_start_".$level]);
		}
		else
		{
		    echo str_replace(array('Increment', 'Level'), array($k, $level), $result["attributes"]["tag_start_".$level]);
		}

		if (
		// на текущей странице нет ссылки
			!$row["active"]

			// влючен пункт "Не отображать ссылку на элемент, если он имеет дочерние пункты"
			&& (!$result["hide_parent_link"] || empty($result["rows"][$row["id"]]))
		)
		{
		    if ($row["othurl"])
		    {
			echo '<a href="'.$row["othurl"].'" target="_blank">';
		    }
		    else
		    {
			echo '<a href="'.BASE_PATH_HREF.$row["link"].'">';
		    }
		}


		// название пункта меню
		echo $row["name"];


		if (
		// на текущей странице нет ссылки
			!$row["active"]

			// влючен пункт "Не отображать ссылку на элемент, если он имеет дочерние пункты"
			&& (!$result["hide_parent_link"] || empty($result["rows"][$row["id"]]))
		)
		{
		    echo '</a>';
		}
		if ($row["active"] && $result["attributes"]["tag_active_end_".$level])
		{
		    echo str_replace(array('Increment', 'Level'), array($k, $level), $result["attributes"]["tag_active_end_".$level]);
		}
		elseif ($row["active_child"] && $result["attributes"]["tag_active_child_end_".$level])
		{
		    echo str_replace(array('Increment', 'Level'), array($k, $level), $result["attributes"]["tag_active_child_end_".$level]);
		}
		else
		{
		    echo str_replace(array('Increment', 'Level'), array($k, $level), $result["attributes"]["tag_end_".$level]);
		}

		if ($result["show_all_level"] || $row["active_child"] || $row["active"])
		{
		    $this->show_menu($result, $row["id"], $level + 1);
		}
		$k++;
	    }
	    echo $result["attributes"]["tag_level_end_".$level];
	}
	return TRUE;
    }

}