<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * News_model
 *
 * Модель модуля "Новости"
 */
class News_model extends Model
{
	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Генерирует данные для списка всех новостей без деления на категории
	 * 
	 * @return array
	 */
	public function list_()
	{
		if ($this->diafan->cat)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		$cache_meta = array(
				"name"     => "list",
				"language" => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"site_id"  => $this->diafan->cid,
				"time"     => $time,
				"year"     => $this->diafan->year,
				"month"    => $this->diafan->month,
			);

		//кеширование
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			if ($this->diafan->year || $this->diafan->month)
			{
				if ($this->diafan->cat)
				{
					include(ABSOLUTE_PATH.'includes/404.php');
				}
				if ($this->diafan->month)
				{
					$month_arr = array(
						'12' => _LANG_DECEMBER,
						'11' => _LANG_NOVEMBER,
						'10' => _LANG_OCTOBER,
						'09' => _LANG_SEPTEMBER,
						'08' => _LANG_AUGUST,
						'07' => _LANG_JULY,
						'06' => _LANG_JUNE,
						'05' => _LANG_MAY,
						'04' => _LANG_APRIL,
						'03' => _LANG_MARCH,
						'02' => _LANG_FEBRUARY,
						'01' => _LANG_JANUARY
					);

					$this->result["titlemodule"] =
						sprintf(
						 _LANG_NEWS_MONTH_YEAR,
						 $month_arr[$this->diafan->month],
						 $this->diafan->year
						);
				}
				else
				{
					$this->result["titlemodule"] =
					sprintf(
						 _LANG_NEWS_YEAR,
						 $this->diafan->year
					);
				}

				$time  = mktime(23, 59, 0, date("m"), date("d"), date("Y"));
				$time1 = mktime(0, 0, 0, ($this->diafan->month ? $this->diafan->month : '1'), 1, ($this->diafan->year ? $this->diafan->year : date("Y")));
				$time2 = mktime(0, 0, 0, ($this->diafan->month ? $this->diafan->month : 12), 31, ($this->diafan->year ? $this->diafan->year : date("Y")));
				$time2 = $time2 < $time ? $time2 : $time;

				////navigation///
				$this->diafan->_paginator->page    = $this->diafan->page;
				$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
				$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
				$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(id) FROM {news} WHERE [act]='1' AND trash='0'"
				                                                  ." AND site_id=%d AND created>=%d AND created<%d",
				                                                  $this->diafan->cid, $time1, $time2);
				$links = $this->diafan->_paginator->get();
				////navigation///

				$result = DB::query_range("SELECT id, created, [name], [anons], timeedit, cat_id, site_id FROM {news} WHERE [act]='1'"
				                          ." AND trash='0' AND site_id=%d AND created>=%d AND created<%d"
				                          ." ORDER BY prior DESC, created DESC, id DESC",
				                          $this->diafan->cid, $time1, $time2,
				                          $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);
			}
			else
			{
				////navigation///
				$this->diafan->_paginator->page    = $this->diafan->page;
				$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
				$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(id) FROM {news} WHERE [act]='1' AND"
				                                                  ." trash='0' AND site_id=%d AND created<%d",
				                                                  $this->diafan->cid, $time);
				$links = $this->diafan->_paginator->get();
				////navigation///

				$result = DB::query_range("SELECT id, created, [name], [anons], timeedit, cat_id, site_id FROM {news}"
				                          ." WHERE [act]='1' AND trash='0' AND site_id=%d"
				                          ." AND created<%d ORDER BY prior DESC, created DESC, id DESC",
				                          $this->diafan->cid, $time, $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);
			}

			$this->result["rows"] = $this->get_elements($result);
			$this->result["path"] = $this->get_path('news');

			$this->result["paginator"] = $links;
			
			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

				$this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';

				$this->result["rows"][$i]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'news', _LANG);

				$this->result["rows"][$i]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'news',  _LANG);

				$this->result["rows"][$i]["date"] = $this->diafan->_useradmin->get($row["date"], 'created', $row["id"], 'news');
			}
		}
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;
	}

	/**
	 * Генерирует данные для первой страницы новостей
	 * 
	 * @return array
	 */
	public function first_page()
	{
		if ($this->diafan->page)
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				"name"     => "first_page",
				"language" => _LANG,
				"time"     => $time,
				"site_id"  => $this->diafan->cid
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$result = DB::query("SELECT id, [name], [anons], timeedit, site_id FROM {news_category}"
			                    ." WHERE [act]='1' AND parent_id=0 AND trash='0' AND site_id=%d"
			                    ." ORDER by sort ASC, id ASC", $this->diafan->cid);

			while ($row = DB::fetch_array($result))
			{
				$this->result["categories"][$row["id"]]["name"] = $row["name"];
				$this->result["categories"][$row["id"]]["anons"] = $row["anons"];

				if (empty($this->result["timeedit"]) || $row["timeedit"] > $this->result["timeedit"])
				{
					$this->result["timeedit"] = $row["timeedit"];
				}

				$this->result["categories"][$row["id"]]["children"] = array();
				$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {news_category}"
				                             ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
				                             ." AND site_id=%d ORDER BY sort ASC, id ASC",
				                             $row["id"], $this->diafan->cid);

				while ($child = DB::fetch_array($result_children))
				{
					$child["link"]  = $this->diafan->_route->link($child["site_id"], 'news', $child["id"]);
					if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
					{
						$child["img"] = $this->diafan->_images->get($child["id"], $child["name"], $this->diafan->module,
						                                            $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
						                                            $child["link"], TRUE);
					}
					unset($child["site_id"]);
					$this->result["categories"][$row["id"]]["children"][] = $child;
				}

				$children = array();
				if ($this->diafan->configmodules("childrens_elements"))
				{
					$children = $this->diafan->get_children($row["id"],"news_category");
				}
				$children[] = $row["id"];

				$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id, e.created FROM {news} AS e"
				                              ." INNER JOIN {news_category_rel} AS c ON e.id=c.element_id"
				                              ." WHERE c.cat_id IN (%s) AND e.[act]='1' AND e.trash='0' AND e.created<'%d'"
				                              ." GROUP BY e.id ORDER BY e.prior DESC, e.created DESC, e.id DESC",
				                              implode(',', $children), $time,
				                              0, $this->diafan->configmodules("count_list")
				                             );
				$this->result["categories"][$row["id"]]["rows"] = $this->get_elements($result_img);

				$this->result["categories"][$row["id"]]["link_all"] = $this->diafan->_route->link($row["site_id"], 'news', $row["id"]);

				if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
				{
					$this->result["categories"][$row["id"]]["img"] =
					$this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module,
					                            $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
					                            $this->result["categories"][$row["id"]]["link_all"],
					                            TRUE);
				}
			}
			
			////navigation///
				$this->diafan->_paginator->page    = $this->diafan->page;
				$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
				$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(id) FROM {news} WHERE [act]='1' AND"
				                                                  ." trash='0' AND site_id=%d AND created<%d AND cat_id='0'",
				                                                  $this->diafan->cid, $time);
				$links = $this->diafan->_paginator->get();
				////navigation///

				$result = DB::query_range("SELECT id, created, [name], [anons], timeedit, cat_id, site_id FROM {news}"
				                          ." WHERE [act]='1' AND trash='0' AND site_id=%d"
				                          ." AND created<%d AND cat_id='0' ORDER BY prior DESC, created DESC, id DESC",
				                          $this->diafan->cid, $time, $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);
				
				$this->result["rows"] = $this->get_elements($result);

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		if (! empty($this->result["categories"]))
		{
			foreach ($this->result["categories"] as $cat => $rows)
			{
				if (! $rows)
					continue;

				$this->result["categories"][$cat]["name"] = $this->diafan->_useradmin->get($rows["name"], 'name', $cat, 'news_category', _LANG);

				$this->result["categories"][$cat]["anons"] = $this->diafan->_useradmin->get($rows["anons"], 'anons', $cat, 'news_category', _LANG);
				foreach ($rows["children"] as $k => $row)
				{
					$this->result["categories"][$cat]["children"][$k]["name"] =
						$this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'news_category', _LANG);

					$this->result["categories"][$cat]["children"][$k]["anons"] =
						$this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'news_category', _LANG);
				}

				foreach ($rows["rows"] as $k => $row)
				{
					$this->result["categories"][$cat]["rows"][$k]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

					$this->result["categories"][$cat]["rows"][$k]["show_rating"] =
						$this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';

					$this->result["categories"][$cat]["rows"][$k]["name"] =
						$this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'news', _LANG);

					$this->result["categories"][$cat]["rows"][$k]["anons"] =
						$this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'news', _LANG);
		
					$this->result["categories"][$cat]["rows"][$k]["date"] = $this->diafan->_useradmin->get($row["date"], 'created', $row["id"], 'news');
				}
			}
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для списка новостей в категории
	 * 
	 * @return array
	 */
	public function list_category()
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				"name"     => "list",
				"cat_id"   => $this->diafan->cat,
				"language" => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"time"     => $time,
				"site_id"  => $this->diafan->cid
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$this->result["level"] = count($this->diafan->get_parents($this->diafan->cat, 'news_category'));
			
			$this->result["path"] = $this->get_path('news');

			$result = DB::query("SELECT id, [name], timeedit, [descr], [keywords], sort, parent_id, [title_meta] FROM {news_category}"
			                    ." WHERE [act]='1' and id=%d AND trash='0' AND site_id=%d"
			                    ." ORDER BY sort ASC, id ASC",
			                    $this->diafan->cat, $this->diafan->cid);

			if (! $row = DB::fetch_array($result))
			{
				include_once(ABSOLUTE_PATH.'includes/404.php');
			}

			$this->result["id"]  = $row["id"];
			if ($this->diafan->configmodules("images_cat"))
			{
				$this->result["img"] = $this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module, 0, 1, TRUE);
			}

			if ($this->diafan->configmodules("childrens_elements"))
			{
				$children = $this->diafan->get_children($this->diafan->cat, "news_category");
			}
			$children[] = $this->diafan->cat;

			////navigation//
			$this->diafan->_paginator->page    = $this->diafan->page;
			$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
			$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
			$this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {news} as e"
			                                                  ." INNER JOIN {news_category_rel} AS c ON e.id=c.element_id"
			                                                  ." AND e.id=c.element_id AND c.cat_id IN (%s)"
			                                                  ." WHERE e.[act]='1' AND e.trash='0' AND e.created<'%d'",
			                                                  implode(',', $children), $time
			                                                 );
			$links = $this->diafan->_paginator->get();
			////navigation///

			$result_img = DB::query_range("SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id, e.created FROM {news} AS e"
			                              ." INNER JOIN {news_category_rel} AS c ON e.id=c.element_id AND c.cat_id IN (%s)"
			                              ." WHERE e.[act]='1' AND e.trash='0' AND created<%d GROUP BY e.id"
			                              ." ORDER BY e.prior DESC, e.created DESC, e.id DESC",
			                              implode(',', $children), $time,
			                              $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);

			$this->result["rows"] = $this->get_elements($result_img);

			$this->result["paginator"] = $links;

			$this->result["titlemodule"] = $row["name"];
			$this->result["edit_titlemodule"] = array("id" => $row["id"], "table" => "news_category");
			$this->result["title_meta"]  = $row["title_meta"];
			$this->result["timeedit"]    = $row["timeedit"];
			$this->result["keywords"]    = $row["keywords"];
			$this->result["descr"]       = $row["descr"];

			$previous = DB::fetch_array(DB::query("SELECT [name], id FROM {news_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                      ." AND (sort<%d OR sort=%d AND id<%d) AND parent_id=%d ORDER BY sort DESC, id DESC LIMIT 1",
			                                      $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if ($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				$this->result["previous"]["id"]   = $previous["id"];
				$this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "news", $previous["id"]);
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id FROM {news_category} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                  ." AND (sort>%d OR sort=%d AND id>%d) AND parent_id=%d ORDER BY sort ASC, id ASC LIMIT 1",
			                                  $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]));
			if ($next)
			{
				$this->result["next"]["text"] = $next["name"];
				$this->result["next"]["id"]   = $next["id"];
				$this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "news", $next["id"]);
			}

			$this->result["children"] = array();
			$result_children = DB::query("SELECT id, [name], [anons], site_id FROM {news_category}"
			                             ." WHERE [act]='1' AND parent_id=%d AND trash='0'"
			                             ." AND site_id=%d ORDER BY sort ASC, id ASC",
			                             $row["id"], $this->diafan->cid);

			while ($child = DB::fetch_array($result_children))
			{
				$child["link"]  = $this->diafan->_route->link($child["site_id"], 'news', $child["id"]);
				if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
				{
					$child["img"] = $this->diafan->_images->get($child["id"], $child["name"], $this->diafan->module,
																$this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
																$child["link"], TRUE);
				}
				unset($child["site_id"]);
				$this->result["children"][] = $child;
			}

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		$text = DB::query_result("SELECT [text] FROM {news_category} WHERE id=%d", $this->diafan->cat);
		$this->result["text"] = $this->diafan->_useradmin->get($text, 'text', $this->diafan->cat, 'news_category', _LANG);

		if ($this->diafan->configmodules("cat") && ! empty($this->diafan->cat) && $this->diafan->configmodules("comments_cat"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

				$this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';

				$this->result["rows"][$i]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'news', _LANG);

				$this->result["rows"][$i]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'news',  _LANG);

				$this->result["rows"][$i]["date"] = $this->diafan->_useradmin->get($row["date"], 'created', $row["id"], 'news');
			}
			if (! empty($this->result["previous"]["text"]))
			{
				$this->result["previous"]["text"] =
					$this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'news_category', _LANG);
			}
			if (! empty($this->result["next"]["text"]))
			{
				$this->result["next"]["text"] =
					$this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'news_category', _LANG);
			}
			foreach ($this->result["children"] as $id => $row)
			{
				$this->result["children"][$id]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'news_category', _LANG);
				$this->result["children"][$id]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'news_category', _LANG);
			}
			foreach ($this->result["path"] as $k => $path)
			{
				if ($k == 0)
					continue;
				$this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'news_category', _LANG);
			}
		}
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;
	}

	/**
	 * Генерирует данные для страницы новости
	 * 
	 * @return array|boolean false
	 */
	public function id()
	{
		//кеширование
		$cache_meta = array(
				"name"     => "show",
				"cat_id"   => $this->diafan->cat,
				"show"     => $this->diafan->show,
				"language" => _LANG,
				"site_id"  => $this->diafan->cid
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

			$row = DB::fetch_array(DB::query("SELECT id, [name], timeedit, created, cat_id, [keywords], [descr], site_id, [title_meta] FROM"
			                                 ." {news} WHERE [act]='1' AND id = %d AND trash='0' AND site_id=%d AND created<%d",
			                                 $this->diafan->show, $this->diafan->cid, $time));
			if (empty($row))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
			}

			if (! $this->diafan->rewrite_module && ($this->diafan->configmodules("cat") && $this->diafan->cat != $row["cat_id"]
			   || ! $this->diafan->configmodules("cat") && $this->diafan->cat))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
			}
			$this->result["id"]     = $this->diafan->show;
			$this->diafan->cat      = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;
			$this->result["cat_id"] = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;

			if ($this->diafan->configmodules("images"))
			{
				$this->result["img"]    = $this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module);
			}

			if ($this->result["cat_id"])
			{
				$this->result["allnews"]["link"] = $this->diafan->_route->link($row["site_id"], "news", $row["cat_id"]);
			}
			else
			{
				$this->result["allnews"]["link"] = $this->diafan->_route->link($row["site_id"]);
			}

			$previous = DB::fetch_array(DB::query("SELECT [name], id, cat_id FROM {news} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                      .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
			                                      ." AND (created<%d OR created=%d AND id<%d) ORDER BY prior DESC, created DESC, id DESC LIMIT 1",
			                                      $this->diafan->cid, $row["created"], $row["created"], $this->diafan->show));
			if ($previous)
			{
				$this->result["previous"]["text"] = $previous["name"];
				if (! $this->diafan->configmodules("cat"))
				{
					$previous["cat_id"] = 0;
				}
				$this->result["previous"]["id"]   = $previous["id"];
				$this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "news", $previous["cat_id"], $previous["id"]);
			}
			$next = DB::fetch_array(DB::query("SELECT [name], id, cat_id FROM {news} WHERE [act]='1' AND trash='0' AND site_id=%d"
			                                  .($this->diafan->configmodules("cat") ? " AND cat_id='".$this->diafan->cat."'" : '')
			                                  ." AND created<%d AND (created>%d OR created=%d AND id>%d)"
			                                  ." ORDER BY prior ASC, created ASC, id ASC LIMIT 1",
			                                  $this->diafan->cid, $time, $row["created"], $row["created"], $this->diafan->show));
			if ($next)
			{
				$this->result["next"]["text"] = $next["name"];
				if (! $this->diafan->configmodules("cat"))
				{
					$next["cat_id"] = 0;
				}
				$this->result["next"]["id"]   = $next["id"];
				$this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "news", $next["cat_id"], $next["id"]);
			}

			$this->result["id"]          = $this->diafan->show;
			$this->result["date"]        = $this->format_date($row['created']);

			$this->result["titlemodule"] = $row["name"];
			$this->result["edit_titlemodule"] = array("id" => $row["id"], "table" => "news");
			$this->result["title_meta"]  = $row["title_meta"];
			$this->result["timeedit"]    = $row["timeedit"];

			$this->result["keywords"]    = $row["keywords"];
			$this->result["descr"]       = $row["descr"];

			$this->result["path"]        = $this->get_path('news');

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		$this->diafan->cat = $this->result["cat_id"];

		$row = DB::fetch_array(DB::query("SELECT [text], [anons] FROM {news} WHERE id=%d", $this->diafan->show));

		$this->result["text"]  = $this->diafan->_useradmin->get($row["text"], 'text', $this->result["id"], 'news', _LANG);
		$this->result["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $this->result["id"], 'news', _LANG);
		$this->result["date"] = $this->diafan->_useradmin->get($this->result["date"], 'created', $this->result["id"], 'news');

		if (! empty($this->result["previous"]["text"]))
		{
			$this->result["previous"]["text"] =
				$this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'news', _LANG);
		}
		if (! empty($this->result["next"]["text"]))
		{
			$this->result["next"]["text"] =
				$this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'news', _LANG);
		}
		foreach ($this->result["path"] as $k => $path)
		{
			if ($k == 0)
				continue;
			$this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'news_category', _LANG);
		}

		if ($this->diafan->configmodules("rating"))
		{
			$this->result["show_rating"] = $this->diafan->_rating->get();
			$this->result["add_rating"] = $this->diafan->_rating->add();
		}

		if ($this->diafan->configmodules("comments"))
		{
			$this->result["comments"] = $this->diafan->_comments->get();
		}

		if ($this->diafan->configmodules("tags"))
		{
			$this->result["tags"] = $this->diafan->_tags->get();
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: блок новостей
	 * 
	 * @param integer $count количество новостей
	 * @param integer $cat_id категория
	 * @param integer $images количество фотографий
	 * @param integer $site_id страница сайта
	 * @return array
	 */
	public function show_block($count, $cat_id, $images, $site_id)
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				"name"     => "block",
				"cat_id"   => $cat_id,
				"count"    => $count,
				"language" => _LANG,
				"time"     => $time,
				"site_id"  => $site_id
			);

		if (! $this->result = $this->diafan->_cache->get($cache_meta, "news"))
		{
			if (! $site_id)
			{
				$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='news' AND trash='0' AND act='1' AND block='0'");
			}
			if ($cat_id && $this->diafan->configmodules("cat", "news"))
			{
				if ($this->diafan->configmodules("childrens_elements", "news"))
				{ 
					$children = $this->diafan->get_children($cat_id, "news_category");
				}
				$children[] = $cat_id;
			}
			$this->result["rows"] = array();
			$result = DB::query_range("SELECT e.id, e.[name],e.[anons], e.cat_id, e.timeedit, e.site_id, e.created FROM {news} AS e"
			                          .($cat_id && $this->diafan->configmodules("cat", "news")
			                            ? " INNER JOIN {news_category_rel} as c ON c.element_id=e.id"
			                              ." AND c.cat_id IN (".implode(',', $children).")"
			                            : ''
			                           )
			                          ." WHERE e.[act]='1' AND e.trash='0' AND e.created<%d"
			                          .($site_id
			                            ? " AND e.site_id=".$site_id
			                            : ''
			                           )
			                          ." GROUP BY e.id"
			                          ." ORDER BY e.prior DESC, e.created DESC, e.id DESC",
			                          $time, 0, $count);
			$this->result["rows"] = $this->get_elements($result, $images);

			if (! empty($this->result["rows"]) && $cat_id && $this->diafan->configmodules("cat","news"))
			{
				$cat = DB::fetch_array(DB::query("SELECT [name], site_id, id FROM {news_category} WHERE id=%d", $cat_id));

				$this->result["name"]     = $cat["name"];
				$this->result["link_all"] = $this->diafan->_route->link($cat["site_id"], 'news', $cat["id"]);
				$this->result["category"] = TRUE;
			}
			elseif (! empty($this->result["rows"]) && $site_id)
			{
				$this->result["name"] = DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $site_id);
				$this->result["link_all"] = $this->diafan->_route->link($site_id);
				$this->result["category"] = FALSE;
			}
			$this->diafan->_cache->save($this->result, $cache_meta, "news");
		}

		if ($this->result["rows"])
		{
			foreach ($this->result["rows"] as $i => $row)
			{
				$this->result["rows"][$i]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'news', _LANG);

				$this->result["rows"][$i]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'news', _LANG);

				$this->result["rows"][$i]["date"] = $this->diafan->_useradmin->get($row["date"], 'created', $row["id"], 'news', _LANG);
			}
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: календарь архива новостей
	 * 
	 * @return array
	 */
	function show_calendar($only_news, $month_hidden, $site_id, $cat_id)
	{
		if (! $site_id && $this->diafan->module == "news")
		{
			$site_id = $this->diafan->cid;
		}
		if ($only_news && $site_id != $this->diafan->cid)
		{
			return FALSE;
		}
		$year_current  = 0;
		$month_current = 0;
		if ($this->diafan->module == "news" && $site_id == $this->diafan->cid)
		{
			$year_current = $this->diafan->year;
			$month_current = $this->diafan->month;
		}

		//кеширование
		$cache_meta = array(
				"name"           => "calendar",
				"month_hidden"   => $month_hidden,
				"language"       => _LANG,
				"site_id"        => $site_id,
				"cat_id"         => $cat_id
			);
		if ($result_cache = $this->diafan->_cache->get($cache_meta, "news"))
		{
			$result_cache["year"]  = $year_current;
			$result_cache["month"] = $month_current;
			return $result_cache;
		}
		if (! $site_id)
		{
			$site_id = DB::query_result("SELECT id FROM {site} WHERE addmodule='news' AND trash='0' AND act='1' AND block='0'");
		}
		if (! $this->diafan->configmodules("cat", "news", $site_id))
		{
			$cat_id = 0;
		}

		$d = DB::query_result("SELECT n.created FROM {news} AS n"
		                      .($cat_id ? "INNER JOIN {news_category_rel} as r ON r.element_id=n.id" : "")
		                      ." WHERE n.[act]='1' AND n.site_id=%d".($cat_id ? " AND r.cat_id=%d" : "")
		                      ." ORDER BY n.created ASC LIMIT 1", $site_id, $cat_id);
		$nyear = 2008;

		if ($d > 0)
		{
			$nyear = date("Y", $d);
		}

		if (date("Y") - $nyear > 10)
		{
			$nyear = 2008;
		}

		for ($Ye = date("Y"); $Ye >= $nyear; $Ye--)
		{
			$result["rows"][$Ye]["year"]["link"] = $this->diafan->_route->link($site_id, "news").'year'.$Ye.'/';
			$result["rows"][$Ye]["year"]["name"] = $Ye;

			$month_arr = array(
				'12' => _LANG_DECEMBER,
				'11' => _LANG_NOVEMBER,
				'10' => _LANG_OCTOBER,
				'09' => _LANG_SEPTEMBER,
				'08' => _LANG_AUGUST,
				'07' => _LANG_JULY,
				'06' => _LANG_JUNE,
				'05' => _LANG_MAY,
				'04' => _LANG_APRIL,
				'03' => _LANG_MARCH,
				'02' => _LANG_FEBRUARY,
				'01' => _LANG_JANUARY
			);

			if (! $month_hidden)
			{
				for ($num = 12; $num > 0; $num--)
				{
					if ($Ye != date("Y") || $num <= date("m"))
					{
						if ($num < 10)
						{
							$num = '0'.$num;
						}

						$val = $month_arr[$num];
						$count_news = DB::query_result("SELECT COUNT(DISTINCT n.id) FROM {news} AS n"
						                               .($cat_id ? "INNER JOIN {news_category_rel} as r ON r.element_id=n.id" : "")
						                               ." WHERE [act]='1' AND trash='0' AND created>%d AND created<=%d AND site_id=%d"
						                               .($cat_id ? " AND r.cat_id=%d" : ""),
						                               mktime(0, 0, 0, $num, 1, $Ye), mktime(0, 0, 0, $num, 31, $Ye), $site_id, $cat_id);
						$result["rows"][$Ye]["months"][$num] = array(
								"link" => $count_news ? $this->diafan->_route->link($site_id, "news").'year'.$Ye.'/month'.$num.'/' : '',
								"name" => $val
							);
					}
				}
			}
		}

		$result["link"] = $this->diafan->_route->link($site_id);

		//сохранение кеша
		$this->diafan->_cache->save($result, $cache_meta, "news");

		$result["year"]  = $year_current;
		$result["month"] = $month_current;
		return $result;
	}

	/**
	 * Форматирует данные о новостях для списка новостей
	 *
	 * @param resource $result результат выполнения SQL-запроса
	 * @param string $images показывать фотографии
	 * @return array
	 */
	public function get_elements($result, $images = 'list')
	{
		if (empty($this->result["timeedit"]))
		{
			$this->result["timeedit"] = '';
		}
		$rows = array();
		while ($row = DB::fetch_array($result))
		{
			if (! $this->diafan->configmodules("cat", "news", $row["site_id"]))
			{
				$row["cat_id"] = 0;
			}
			if ($row["timeedit"] < $this->result["timeedit"])
			{
				$this->result["timeedit"] = $row["timeedit"];
			}
			unset($row["timeedit"]);

			$row["link"] = $this->diafan->_route->link($row["site_id"], "news", $row["cat_id"], $row["id"]);
			unset($row["cat_id"]);

			if ($this->diafan->configmodules("images", "news", $row["site_id"]))
			{
				if ($images === 'list' && $this->diafan->configmodules("list_img", "news", $row["site_id"]))
				{
					$row["img"]  = $this->diafan->_images->get($row["id"], $row["name"], 'news',
					               $this->diafan->configmodules("list_img", "news", $row["site_id"]) == 1 ? 1 : 0,
					               $row["link"]);
				}
				elseif ($images != 'list' && $images)
				{
					$row["img"]  = $this->diafan->_images->get($row["id"], $row["name"], 'news', $images, $row["link"]);
				}
			}
			$row["date"] = $this->format_date($row['created'], "news", $row["site_id"]);
			unset($row["created"]);

			$rows[] = $row;
		}
		return $rows;
	}
}