<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * News_view
 *
 * Шаблон модуля "Новости"
 */
class News_view extends View
{
	/**
	 * Функция вывода списка новостей в том случае, 
	 * если в конфигурации модуля отключен (по умолчанию) параметр "Использовать категории". 
	 * Если параметр "Использовать категории" включен,
	 * список новостей формируется с группировкой по категориям в функции first_page() ниже
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean true
	 */
	public function list_($result)
	{
		echo '<div class="news_list">';

		//вывод описания текущей категории новостей
		if (! empty($result["text"]))
		{
			echo '<div class="news_cat_text">'.$result["text"].'</div>';
		}

		//вывод изображений текущей категории
		if (! empty($result["img"]))
		{
			echo '<div class="news_cat_all_img">';
			foreach ($result["img"] as $img)
			{
				switch($img["type"])
				{
					case 'animation':
						echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$result["id"].'news]">';
						break;
					case 'big_image':
						echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
						break;
					default:
						echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
						break;
				}
				echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
				.'</a> ';
			}
			echo '</div>';
		}

		//вывод подкатегорий
		if (! empty($result["children"]))
		{
			foreach ($result["children"] as $child)
			{
				echo '<div class="news_cat_link">';

				//изображение подкатегории
				if (! empty($child["img"]))
				{
					echo '<div class="news_cat_img">';
					foreach ($child["img"] as $img)
					{
						switch($img["type"])
						{
							case 'animation':
								echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$child["id"].'news]">';
								break;
							case 'big_image':
								echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
								break;
							default:
								echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
								break;
						}
						echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
						.'</a> ';
					}
					echo '</div>';
				}

				//название и ссылка подкатегории
				echo '<a href="'.BASE_PATH_HREF.$child["link"].'">'.$child["name"].'</a>';

				//краткое описание подкатегории
				if ($child["anons"])
				{
					echo '<div class="news_cat_anons">'.$child["anons"].'</div>';
				}
				echo '</div>';
			}
		}
		
		

		//вывод списка новостей
		if (! empty($result["rows"]))
		{
			foreach ($result["rows"] as $row)
			{
				echo '<div class="news">';
				
				//вывод даты новости
				if (! empty($row['date']))
				{
					echo '<div class="news_date">'.$row["date"]."</div>";
				}
				
				//вывод названия и ссылки на новость
				echo '<div class="news_name">';
				echo '<a href="'.BASE_PATH_HREF.$row["link"].'">'.$row["name"].'</a>';
					//вывод рейтинга новости за названием, если рейтинг подключен
					if (! empty($row["show_rating"])) echo ' ' .$row["show_rating"];
				echo '</div>';
				
				//вывод изображений новости
				if (! empty($row["img"]))
				{
					echo '<div class="news_img">';
					foreach ($row["img"] as $img)
					{
						switch($img["type"])
						{
							case 'animation':
								echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$row["id"].'news]">';
								break;
							case 'big_image':
								echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
								break;
							default:
								echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
								break;
						}
						echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
						.'</a> ';
					}
					echo '</div>';
				}
				
				//вывод анонса новостей
				if (! empty($row["anons"]))
				{
					echo '<div class="news_anons">'.$row["anons"].'</div>';
				}
				
				
				//вывод прикрепленных тегов к новости
				if (! empty($row["tags"]))
				{
					echo $row["tags"];
				}
				
				echo '</div>';
			}
			echo '<div class="clear"></div>';
		}

		//вывод постраничная навигация в конце списка новостей
		if (! empty($result["paginator"]))
		{
			echo $result["paginator"];
		}

		//вывод ссылок на предыдущую и последующую категории
		if (! empty($result["previous"]) || ! empty($result["next"]))
		{
			echo '<div class="previous_next_links">';
			if (! empty($result["previous"]))
			{
				echo '<div class="previous_link"><a href="'.BASE_PATH_HREF.$result["previous"]["link"].'">&larr; '.$result["previous"]["text"].'</a></div>';
			}
			if (! empty($result["next"]))
			{
				echo '<div class="next_link"><a href="'.BASE_PATH_HREF.$result["next"]["link"].'">'.$result["next"]["text"].' &rarr;</a></div>';
			}
			echo '</div>';
		}

		//вывод комментариев к категориям, если они подключены в настройках
		if (! empty($result["comments"]))
		{
			echo $result["comments"];
		}
		
		echo '</div>';

		return TRUE;
	}

	/**
	 * Шаблон первой страницы модуля "Новости", если подключен параметр "Использовать категории"
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean
	 */
	public function first_page($result)
	{
	    
		if (!empty($result["categories"]))
		{

		//вывод категории
		foreach ($result["categories"] as $cat_id => $cat)
		{
			echo '<div class="news_list">';

			//вывод названия категории новостей
			echo '<h2>'.$cat["name"].'</h2>';

			//вывод изображений категории
			if (! empty($cat["img"]))
			{
				echo '<div class="news_cat_img">';
				foreach ($cat["img"] as $img)
				{
					switch($img["type"])
					{
						case 'animation':
							echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$cat_id.'news]">';
							break;
						case 'big_image':
							echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
							break;
						default:
							echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
							break;
					}
					echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
					.'</a> ';
				}
				echo '</div>';
			}

			//вывод краткого описания категории
			if (! empty($cat["anons"]))
			{
				echo '<div class="news_cat_anons">'.$cat["anons"].'</div>';
			}

			//вывод подкатегории
			if (! empty($cat["children"]))
			{
				foreach ($cat["children"] as $child)
				{
					echo '<div class="news_cat_link">';

					//изображения подкатегории
					if (! empty($child["img"]))
					{
						echo '<div class="news_cat_img">';
						foreach ($child["img"] as $img)
						{
							switch($img["type"])
							{
								case 'animation':
									echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$child["id"].'news]">';
									break;
								case 'big_image':
									echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
									break;
								default:
									echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
									break;
							}
							echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
							.'</a> ';
						}
						echo '</div>';
					}

					//название и ссылка подкатегории
					echo '<a href="'.BASE_PATH_HREF.$child["link"].'">'.$child["name"].'</a>';

					//краткое описание подкатегории
					if (! empty($child["anons"]))
					{
						echo '<div class="news_cat_anons">'.$child["anons"].'</div>';
					}
					echo '</div>';
				}
			}

			//вывод нескольких новостей из категории
			if ($cat["rows"])
			{
				foreach ($cat["rows"] as $row)
				{
					echo '<div class="news">';
					
					//вывод даты новости
					if (! empty($row['date']))
					{
						echo '<div class="news_date">'.$row["date"]."</div>";
					}
					
					//вывод названия и ссылки на новость
					echo '<div class="news_name">';
					echo '<a href="'.BASE_PATH_HREF.$row["link"].'">'.$row["name"].'</a>';
						//вывод рейтинга новости за названием, если рейтинг подключен
						if (! empty($row["show_rating"])) echo ' ' .$row["show_rating"];
					echo '</div>';
					
					//вывод изображений новости
					if (! empty($row["img"]))
					{
						echo '<div class="news_img">';
						foreach ($row["img"] as $img)
						{
							switch($img["type"])
							{
								case 'animation':
									echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$row["id"].'news]">';
									break;
								case 'big_image':
									echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
									break;
								default:
									echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
									break;
							}
							echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
							.'</a> ';
						}
						echo '</div>';
					}
					
					//вывод анонса новости
					if (! empty($row["anons"]))
					{
						echo '<div class="news_anons">'.$row["anons"].'</div>';
					}					
					
					//теги новости
					if (! empty($row["tags"]))
					{
						echo $row["tags"];
					}
					
					echo '</div>';
				}
			}
			//ссылка на все новости в категории
			if ($cat["link_all"])
			{
				echo  '<div class="show_all"><a href="'.BASE_PATH_HREF.$cat["link_all"].'">'
				.$this->_('SHOW_ALL_NEWS_IN_CATEGORY').' '.$this->_('LQUOT').$cat["name"].$this->_('RQUOT')
				.'</a></div>';
			}
			echo '</div>';
		}
		}
		
		//вывод новостей, которые не принадлежат никаким категориям
		if (! empty($result["rows"]))
		{
		    echo '<br></br>';
			foreach ($result["rows"] as $row)
			{
				echo '<div class="news">';
				
				//дата новости
				if (! empty($row['date']))
				{
					echo '<div class="news_date">'.$row["date"]."</div>";
				}
				
				//вывод названия и ссылки на новость
				echo '<div class="news_name">';
				echo '<a href="'.BASE_PATH_HREF.$row["link"].'">'.$row["name"].'</a>';
					//вывод рейтинга новости за названием, если рейтинг подключен
					if (! empty($row["show_rating"])) echo ' ' .$row["show_rating"];
				echo '</div>';
				
				//изображения новости
				if (! empty($row["img"]))
				{
					echo '<div class="news_img">';
					foreach ($row["img"] as $img)
					{
						switch($img["type"])
						{
							case 'animation':
								echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$row["id"].'news]">';
								break;
							case 'big_image':
								echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
								break;
							default:
								echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
								break;
						}
						echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
						.'</a> ';
					}
					echo '</div>';
				}
				
				//анонс новости
				if (! empty($row["anons"]))
				{
					echo '<div class="news_anons">'.$row["anons"].'</div>';
				}

				//теги новости
				if (! empty($row["tags"]))
				{
					echo $row["tags"];
				}

				echo '</div>';
			}
			echo '<div class="clear"></div>';
		}

		//постраничная навигация
		if (! empty($result["paginator"]))
		{
			echo $result["paginator"];
		}

		return TRUE;
	}

	/**
	 * Шаблон оформления страницы с отдельной новостью
	 * 
	 * @param array $row данные, сгенерированные в моделе
	 * @return boolean true
	 */
	public function id($row)
	{
		echo '<div class="news_id">';

		//вывод даты новости
		if (! empty($row["date"]))
		{
			echo '<div class="news_date">'.$row["date"]."</div>";
		}		
		
		//изображения новости
		if (! empty($row["img"]))
		{
			echo '<div class="news_all_img">';
			foreach ($row["img"] as $img)
			{
				switch($img["type"])
				{
					case 'animation':
						echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$row["id"].'news]">';
						break;
					case 'big_image':
						echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
						break;
					default:
						echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
						break;
				}
				echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
				.'</a><br>';
			}
			echo '</div>';
		}
		
		//вывод анонса новости
		if ($row['anons'])
		{
			echo '<div class="news_anons">'.$row['anons'].'</div>';
		}

		//вывод основного текста новости
		echo '<div class="news_text">'.$row['text'].'</div>';

		//вывод тегов к новости
		if (! empty($row["tags"]))
		{
			echo $row["tags"];
		}
		
		//рейтинг новости
		if (! empty($row["show_rating"]))
		{
			echo $row["show_rating"];
		}
		
		//форма оценки рейтинга новости
		if (! empty($row["add_rating"]))
		{
			echo $row["add_rating"];
		}
		
		//комментарии к новости
		if (! empty($row["comments"]))
		{
			echo $row["comments"];
		}
		
		
		//ссылки на предыдущую и последующую новость
		if (! empty($row["previous"]) || ! empty($row["next"]))
		{
			echo '<div class="previous_next_links">';
			if (! empty($row["previous"]))
			{
				echo '<div class="previous_link"><a href="'.BASE_PATH_HREF.$row["previous"]["link"].'">&larr; '.$row["previous"]["text"].'</a></div>';
			}
			if (! empty($row["next"]))
			{
				echo '<div class="next_link"><a href="'.BASE_PATH_HREF.$row["next"]["link"].'">'.$row["next"]["text"].' &rarr;</a></div>';
			}
			echo '</div>';
		}

		//ссылки на все новости
		if (! empty($row["allnews"]))
		{
			echo '<div class="show_all"><a href="'.BASE_PATH_HREF.$row["allnews"]["link"].'">'.$this->_('BACK_TO_LIST').'</a></div>';
		}
		
		echo '</div>';

		return TRUE;
	}

	/**
	 * Оформление блока новостей, выводимого тегом show_block
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean
	 */
	public function show_block($result)
	{
		if (empty($result["rows"]))
		{
			return FALSE;
		}
		
		echo '<div class="news_block">';
		
		//заголовок блока
		if (! empty($result["name"]))
		{
			echo '<h2>'.$result["name"].'</h2>';
		}
		
		//новости
		foreach ($result["rows"] as $row)
		{
			echo '
			<div class="news">';
			
			//дата новости
			if (! empty($row["date"]))
			{
				echo '<div class="news_date">'.$row["date"].'</div>';
			}
			
			//название и ссылка новости
			echo '<div class="news_name"><a href="'.BASE_PATH_HREF.$row["link"].'">'.$row['name'].'</a></div>';
			
			//изображения новости
			if (! empty($row["img"]))
			{
				echo '<div class="news_img">';
				foreach ($row["img"] as $img)
				{
					switch($img["type"])
					{
						case 'animation':
							echo '<a href="'.BASE_PATH.$img["link"].'" rel="prettyPhoto[gallery'.$row["id"].'news]">';
							break;
						case 'big_image':
							echo '<a href="'.BASE_PATH.$img["link"].'" rel="big_image" width="'.$img["link_width"].'" height="'.$img["link_height"].'">';
							break;
						default:
							echo '<a href="'.BASE_PATH_HREF.$img["link"].'">';
							break;
					}
					echo '<img src="'.$img["src"].'" width="'.$img["width"].'" height="'.$img["height"].'" alt="'.$img["alt"].'" title="'.$img["title"].'">'
					.'</a> ';
				}
				echo '</div>';
			}
			
			//анонс новости
			echo '<div class="news_anons">'.$row['anons'].'</div>';
			echo '</div>';
		}

		//ссылка на все новости
		if (! empty($result["link_all"]))
		{
			echo '<div class="show_all"><a href="'.BASE_PATH_HREF.$result["link_all"].'">';
			if ($result["category"])
			{
				echo $this->_('SHOW_ALL_NEWS_IN_CATEGORY').' '.$this->_('LQUOT').$result["name"].$this->_('RQUOT');
			}
			else
			{
				echo $this->_('ALL_NEWS');
			}
			echo '</a></div>';
		}
		
		echo '</div>';

		return TRUE;
	}

	/**
	 * Шаблонная функция: календарь архива новостей 
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean true
	 */
	public function show_calendar($result)
	{
		echo '<div class="news_block">';
		echo '<h2>'.$this->_('ARCHIVE').'</h2>';
		
		foreach ($result["rows"] as $key => $values)
		{
			if ($result["year"] == $key && ! $result["month"])
			{
				echo '<div class="news_year_current">'.$values["year"]["name"].'</div>';
			}
			else
			{
				echo '<div class="news_year"><a href="'.BASE_PATH_HREF.$values["year"]["link"].'">'.$values["year"]["name"].'</a></div>';
			}
			if (($result["year"] == $key || ! $result["year"] && date("Y") == $key) && ! empty($values["months"]))
			{
				foreach ($values["months"] as $keym => $month)
				{
					if ($result["month"] == $keym)
					{
						echo '<div class="news_month_current">'.$month["name"].'</div>';
					}
					elseif (! $month["link"])
					{
						echo '<div class="news_month">'.$month["name"].'</div>';
					}
					else
					{
						echo '<div class="news_month"><a href="'.BASE_PATH_HREF.$month["link"].'">'.$month["name"].'</a></div>';
					}
				}
			}
		}
		
		echo '</div>';

		return TRUE;
	}
}