<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Photo_view
 *
 * Шаблон модуля "Фотогалерея"
 */
class Photo_view extends View
{ 
	/**
	 * Оформление списка фотографий, если зайти в категорию
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean true
	 */
	public function list_($result)
	{
		echo '<div class="photo_list">';

		//вывод описания текущей категории
		if (! empty($result["text"]))
		{
			echo '<div class="photo_cat_text">'.$result["text"].'</div>';
		}

		//вывод подкатегорий
		if (! empty($result["children"]))
		{
			foreach ($result["children"] as $child)
			{
				echo '<div class="photo_cat_link">';

				//вывод названий и ссылок на подкатегории
				echo '<a href="'.BASE_PATH_HREF.$child["link"].'">'.$child["name"].'</a>';

				//краткое описание подкатегории
				if ($child["anons"])
				{
					echo '<div class="photo_cat_anons">'.$child["anons"].'</div>';
				}
				echo '</div>';
			}
		}

		//вывод списка фотографий
		if (! empty($result["rows"]))
		{
			foreach ($result["rows"] as $row)
			{
				echo '<div class="photo">';

				//вывод рейтинга фотографии
				if (! empty($row["show_rating"]))
				{
					echo $row["show_rating"];
				}
				
				//вывод маленького изображения
				if (! empty($row["img"]))
				{
					echo '<div class="photo_img">';
					switch($row["img"]["type"])
					{
						case 'animation':
							echo '<a href="'.BASE_PATH.$row["img"]["link"].'" rel="prettyPhoto[galleryphoto]">';
							break;
						case 'big_image':
							echo '<a href="'.BASE_PATH.$row["img"]["link"].'" rel="big_image" width="'.$row["img"]["link_width"].'" height="'.$row["img"]["link_height"].'">';
							break;
						default:
							echo '<a href="'.BASE_PATH_HREF.$row["img"]["link"].'">';
							break;
					}
					echo '<img src="'.$row["img"]["src"].'" width="'.$row["img"]["width"].'" height="'.$row["img"]["height"]
					.'" alt="'.$row["img"]["alt"].'" title="'.$row["img"]["title"].'">'
					.'</a></div>';
				}

				//вывод названия и, если используется, ссылки на отдельную страницу фотографии
				if ($row["name"])
				{
					echo '<div class="photo_name">';
					if ($row["link"])
					{
						echo '<a href="'.BASE_PATH_HREF.$row["link"].'">';
					}
					echo $row["name"];
					if ($row["link"])
					{
						echo '</a>';
					}
					echo '</div>';
				}

				//вывод краткого описания фотографии
				if (! empty($row["anons"]))
				{
					echo '<div class="photo_anons">'.$row["anons"].'</div>';
				}
				
				echo '</div>';
			}
			echo '<div class="clear"></div>';
		}

		//вывод комментариев к категориям, если подключены в настройках
		if (! empty($result["comments"]))
		{
			echo $result["comments"];
		}
		
		//вывод постраничной навигации
		if (! empty($result["paginator"]))
		{
			echo $result["paginator"];
		}

		//ссылки на предыдущую и последующую категории
		if (! empty($result["previous"]) || ! empty($result["next"]))
		{
			echo '<div class="previous_next_links">';
			
			if (! empty($result["previous"]))
			{
				echo '<div class="previous_link"><a href="'.BASE_PATH_HREF.$result["previous"]["link"].'">&larr; '.$result["previous"]["text"].'</a></div>';
			}
			
			if (! empty($result["next"]))
			{
				echo '<div class="next_link"><a href="'.BASE_PATH_HREF.$result["next"]["link"].'">'.$result["next"]["text"].' &rarr;</a></div>';
			}
			
			echo '</div>';
		}

		echo '</div>';

		return TRUE;
	}

	/**
	 * Шаблон первой страницы фотогалереи, где выводятся все категории и по несколько фотографий из каждой
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean
	 */
	public function first_page($result)
	{
		if (empty($result["categories"]))
			return FALSE;

		//вывод категорий
		foreach ($result["categories"] as $cat_id => $cat)
		{
			echo '<div class="photo_list">';

			//название категории
			echo '<h2>'.$cat["name"].'</h2>';

			//краткое описание категории
			if (! empty($cat["anons"]))
			{
				echo '<div class="photo_cat_anons">'.$cat["anons"].'</div>';
			}

			//подкатегории
			if (! empty($cat["children"]))
			{
				foreach ($cat["children"] as $child)
				{
					echo '<div class="photo_cat_link">';

					//название и ссылка подкатегории
					echo '<a href="'.BASE_PATH_HREF.$child["link"].'">'.$child["name"].'</a>';

					//краткое описание подкатегории
					if (! empty($child["anons"]))
					{
						echo '<div class="photo_cat_anons">'.$child["anons"].'</div>';
					}
					echo '</div>';
				}
			}
			
			//вывод нескольких фотографий из текущей категории (задается в настройках модуля)
			if ($cat["rows"])
			{
				foreach ($cat["rows"] as $row)
				{
					echo '<div class="photo">';

					//изображение
					if (! empty($row["img"]))
					{
						echo '<div class="photo_img">';
						switch($row["img"]["type"])
						{
							case 'animation':
								echo '<a href="'.BASE_PATH.$row["img"]["link"].'" rel="prettyPhoto[galleryphoto]">';
								break;
							case 'big_image':
								echo '<a href="'.BASE_PATH.$row["img"]["link"].'" rel="big_image" width="'.$row["img"]["link_width"].'" height="'.$row["img"]["link_height"].'">';
								break;
							default:
								echo '<a href="'.BASE_PATH_HREF.$row["img"]["link"].'">';
								break;
						}
						echo '<img src="'.$row["img"]["src"].'" width="'.$row["img"]["width"].'" height="'.$row["img"]["height"]
						.'" alt="'.$row["img"]["alt"].'" title="'.$row["img"]["title"].'">'
						.'</a></div>';
					}

					//название и, если используется, ссылка на отдельную страницу фотографии
					if ($row["name"])
					{
						echo '<div class="photo_name">';
						if ($row["link"])
						{
							echo '<a href="'.BASE_PATH_HREF.$row["link"].'">';
						}
						echo $row["name"];
						if ($row["link"])
						{
							echo '</a>';
						}
						echo '</div>';
					}

					//краткое описание фотографии
					if (! empty($row["anons"]))
					{
						echo '<div class="photo_anons">'.$row["anons"].'</div>';
					}

					//рейтинг фотографии
					if (! empty($row["show_rating"]))
					{
						echo $row["show_rating"];
					}
					
					echo '</div>';
				}
			}

			//ссылка на все фотографии в категории
			if ($cat["link_all"])
			{
				echo '<div class="show_all"><a href="'.BASE_PATH_HREF.$cat["link_all"].'">'
				.$this->_('SHOW_ALL_PHOTOS_IN_CATEGORY').' '.$this->_('LQUOT').$cat["name"].$this->_('RQUOT')
				.'</a></div>';
			}
			
			echo '</div>';
		}

		return TRUE;
	}

	/**
	 * Оформление страницы с отдельной фотографией
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean true
	 */
	public function id($row)
	{
			echo '<div class="photo_id">';

		//рейтинг фотографии
		if (! empty($row["show_rating"]))
		{
			echo $row["show_rating"];
		}

		//форма оценки фотографии
		if (! empty($row["add_rating"]))
		{
			echo $row["add_rating"];
		}

		//изображение и ссылка на следующее фото
		if (! empty($row["img"]))
		{
			echo '<div class="photo_img">';
			echo (!empty($row["next"])?'<a href="'.BASE_PATH_HREF.$row["next"]["link"].'">':'');
			echo '<img src="'.$row["img"]["src"].'" width="'.$row["img"]["width"].'" height="'.$row["img"]["height"].'"	alt="'.$row["img"]["alt"].'" title="'.$row["img"]["title"].'">';
			echo (!empty($row["next"])?'</a>':'');
			echo '</div>';
		}

		//краткое описание фотографии
		if ($row["anons"])
		{
			echo '<div class="photo_anons">'.$row["anons"].'</div>';
		}

		//полное описание фотографии
		echo '<div class="photo_text">'.$row["text"].'</div>';

		//теги фотографии
		if (! empty($row["tags"]))
		{
			echo $row["tags"];
		}
		
		//комментарии к фотографии
		if (! empty($row["comments"]))
		{
			echo $row["comments"];
		}		

		//ссылки на предыдущую и последующую фотографии
		if (! empty($row["previous"]) || ! empty($row["next"]))
		{
			echo '<div class="previous_next_links">';
			if (! empty($row["previous"]))
			{
				echo '<div class="previous_link"><a href="'.BASE_PATH_HREF.$row["previous"]["link"].'">&larr; '.$row["previous"]["text"].'</a></div>';
			}
			if (! empty($row["next"]))
			{
				echo '<div class="next_link"><a href="'.BASE_PATH_HREF.$row["next"]["link"].'">'.$row["next"]["text"].' &rarr;</a></div>';
			}
			echo '</div>';
		}

			echo '</div>';

		return TRUE;
	}

	/**
	 * Оформление блока фотографий, выводимый тегом show_block module=photo
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean
	 */
	public function show_block($result)
	{
		if (empty($result["rows"]))
		{
			return FALSE;
		}
		
		echo '<div class="photo_block">';

		//заголовок блока
		if (! empty($result["name"]))
		{
			echo '<h2>'.$result["name"].'</h2>';
		}

		//фотографии
		foreach ($result["rows"] as $row)
		{
			echo '<div class="photo">';

			//изобаражение
			if (! empty($row["img"]))
			{
				echo '<div class="photo_img">';
				switch($row["img"]["type"])
				{
					case 'animation':
						echo '<a href="'.BASE_PATH.$row["img"]["link"].'" rel="prettyPhoto[galleryphotoblock]">';
						break;
					case 'big_image':
						echo '<a href="'.BASE_PATH.$row["img"]["link"].'" rel="big_image" width="'.$row["img"]["link_width"].'" height="'.$row["img"]["link_height"].'">';
						break;
					default:
						echo '<a href="'.BASE_PATH_HREF.$row["img"]["link"].'">';
						break;
				}
				echo '<img src="'.$row["img"]["src"].'" width="'.$row["img"]["width"].'" height="'.$row["img"]["height"]
				.'" alt="'.$row["img"]["alt"].'" title="'.$row["img"]["title"].'">'
				.'</a></div>';
			}

			//название и ссылка фотографии
			if ($row["name"])
			{
				echo '<div class="photo_name">';
				if ($row["link"])
				{
					echo '<a href="'.BASE_PATH_HREF.$row["link"].'">';
				}
				echo $row["name"];
				if ($row["link"])
				{
					echo '</a>';
				}
				echo '</div>';
			}

			//краткое описание фотографии
			if (! empty($row["anons"]))
			{
				echo '<div class="photo_anons">'.$row["anons"].'</div>';
			}
			echo '</div>';
		}

		//ссылка на все фотографии
		if (! empty($result["link_all"]))
		{
			echo '<div class="show_all"><a href="'.BASE_PATH_HREF.$result["link_all"].'">';
			if ($result["category"])
			{
				echo $this->_('SHOW_ALL_PHOTOS_IN_CATEGORY').' '.$this->_('LQUOT').$result["name"].$this->_('RQUOT');
			}
			else
			{
				echo $this->_('ALL_PHOTOS');
			}
			echo '</a></div>';
		}
		
		echo '</div>';

		return TRUE;
	}
	/**
	 * Шаблон списка фотографий для модуля "Теги"
	 * 
	 * @param array $result данные, сгенерированные в моделе
	 * @return boolean true
	 */
	public function list_tags($result)
	{
		//название и описание категории
		echo '<div class="tags_list">';

		//фотографии
		if (! empty($result["rows"]))
		{
			foreach ($result["rows"] as $row)
			{
				echo '<div class="photo">';

				//изобаражение
				if (! empty($row["img"]))
				{
					echo '<div class="photo_img">';
					switch($row["img"]["type"])
					{
						case 'animation':
							echo '<a href="'.BASE_PATH.$row["img"]["link"].'" rel="prettyPhoto[galleryphototags]">';
							break;
						case 'big_image':
							echo '<a href="'.BASE_PATH.$row["img"]["link"].'" rel="big_image" width="'.$row["img"]["link_width"].'" height="'.$row["img"]["link_height"].'">';
							break;
						default:
							echo '<a href="'.BASE_PATH_HREF.$row["img"]["link"].'">';
							break;
					}
					echo '<img src="'.$row["img"]["src"].'" width="'.$row["img"]["width"].'" height="'.$row["img"]["height"]
					.'" alt="'.$row["img"]["alt"].'" title="'.$row["img"]["title"].'">'
					.'</a></div>';
				}

				//название и ссылка фотографии
				if ($row["name"])
				{
					echo '<div class="photo_name">';
					if ($row["link"])
					{
						echo '<a href="'.BASE_PATH_HREF.$row["link"].'">';
					}
					echo $row["name"];
					if ($row["link"])
					{
						echo '</a>';
					}
					echo '</div>';
				}

				//краткое описание фотографии
				if (! empty($row["anons"]))
				{
					echo '<div class="photo_anons">'.$row["anons"].'</div>';
				}

				//рейтинг фотографии
				if (! empty($row["show_rating"]))
				{
					echo $row["show_rating"];
				}

				//теги фотографии
				if (! empty($row["tags"]))
				{
					echo $row["tags"];
				}

				echo '</div>';

				}
			echo '<div class="clear"></div>';
		}

		echo '</div>';

		return TRUE;
	}
}