<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Tags_admin_ajax
 * 
 * Обработка Ajax-запросов при работе с тегами в административной части
 */
class Tags_admin_ajax
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array настройки отображения
	 */
	public $config_module;

	/**
	 * @var string таблица в базе данных
	 */
	public $table;

	/**
	 * @var array результаты, передаваемы Ajaxом
	 */
	private $result;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($frame)
	{
		$this->diafan = $frame->diafan;
		$this->config_module  = $frame->config_module;
		$this->table  = $frame->table;

		// Прошел ли пользователь проверку идентификационного хэша
		if (! $this->diafan->_user->checked)
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$this->result["hash"] = $this->diafan->_user->get_hash();

		if (! empty($_POST["action"]))
		{
			switch($_POST["action"])
			{
				case 'upload':
					$this->upload();
					break;

				case 'delete':
					$this->delete();
					break;

				case 'search':
					$this->search();
					break;
			}
		}
	}

	/**
	 * Загружает изображение
	 * 
	 * @return void
	 */
	private function upload()
	{
		if ($this->diafan->admin_roles('save'))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_SAVE;
			$this->send_json();
		}

		if (! $_POST["tag"])
		{
			$this->result["error"] = _ADMIN_TAGS_FIELD_NULL;
			$this->send_json();
		}
		if (! $tags_name_id = DB::query_result("SELECT id FROM {tags_name} WHERE [name]='%h'", trim($_POST["tag"])))
		{
			DB::query("INSERT INTO {tags_name} ([name]) VALUES ('%h')", trim($_POST["tag"]));
			$tags_name_id = DB::next_id("tags_name");
			DB::query("UPDATE {tags_name} SET `sort`=`id` WHERE id=%d", $tags_name_id);
		}

		$this->result["id"] = $this->get_id_element();

		if (DB::query_result("SELECT COUNT(id) FROM {tags} WHERE module_name='%h' AND element_id='%d' AND tags_name_id='%d'",
		                    $this->diafan->rewrite,
		                    $this->result["id"],
		                    $tags_name_id
		))
		{
			$this->result["error"] = _ADMIN_TAGS_TAG_EXISTS;
			$this->send_json();
		}

		DB::query("INSERT INTO {tags} (module_name, element_id, tags_name_id) VALUES ('%h', %d, '%d')",
		          $this->diafan->rewrite,
		          $this->result["id"],
		          $tags_name_id
		);
		$tags_id = DB::next_id("tags");
		foreach ($this->diafan->language_all as $l => $langarr)
		{
			if (DB::query_result("SELECT `act".$langarr[0]."` FROM {".$this->diafan->rewrite."} WHERE id=%d", $this->result["id"]))
			{
				DB::query("UPDATE {tags} SET `act".$langarr[0]."`='1' WHERE id=%d", $tags_id);
			}
		}

		$this->diafan->_cache->delete("", "tags");

		include_once(ABSOLUTE_PATH.'modules/tags/admin/tags.admin.view.php');

		$this->result["data"]   = Tags_admin_view::show($this, $this->result["id"]);
		$this->result["target"] = ".tags_container";

		$this->send_json();
		return TRUE;
	}

	/**
	 * Удаляет изображение
	 * 
	 * @return void
	 */
	private function delete()
	{
		if ($this->diafan->admin_roles('del'))
		{
			$this->result["error"] = _ADMIN_NO_RULES_FOR_DELETE;
			$this->send_json();
		}

		$tag_id = intval($_POST['tag_id']);
		if (! empty($tag_id))
		{
			$row = DB::fetch_array(DB::query("SELECT element_id, tags_name_id FROM {tags} WHERE module_name='%h' AND id='%d'", $this->diafan->rewrite, $tag_id));
			if (! $row)
			{
				$this->result["error"] = 'ERROR';
				$this->send_json();
			}
		}

		DB::query("DELETE FROM {tags} WHERE module_name='%h' AND id='%d'", $this->diafan->rewrite, $tag_id);
		if (! DB::query_result("SELECT COUNT(*) FROM {tags} WHERE tags_name_id=%d", $row["tags_name_id"]))
		{
			DB::query("DELETE FROM {tags_name} WHERE id='%d'", $row["tags_name_id"]);
		}

		$this->diafan->_cache->delete("", "tags");

		include_once(ABSOLUTE_PATH.'modules/tags/admin/tags.admin.view.php');

		$this->result["data"]   = Tags_admin_view::show($this, $row["element_id"]);
		$this->send_json();
		return TRUE;
	}

	/**
	 * Редактирует изображение
	 * 
	 * @return boolean true
	 */
	private function search()
	{
		$mes = '';

		//максимальный и минимальный размеры текста в em
		$max = 3;
		$min = 0.9;

		$maxr = 0;
		$minr = 10;
		$rows = array();

		$res = DB::query("SELECT id, [name] FROM {tags_name} WHERE trash='0'"
		                 .(empty($_POST["new"])
		                   ? " AND id NOT IN"
		                     ." (SELECT tags_name_id FROM {tags} WHERE module_name='%h' AND element_id='%d')"
		                   : "")
		                 ." ORDER BY sort ASC",
		                 $this->diafan->rewrite, $_POST["element_id"]);
		while ($row = DB::fetch_array($res))
		{
			$row["size"] = DB::query_result("SELECT COUNT(*) FROM {tags} WHERE tags_name_id='%d' and trash='0'", $row["id"]);
			$maxr = $maxr < $row["size"] ? $row["size"] : $maxr;
			$minr = $minr > $row["size"] ? $row["size"] : $minr;
			$rows[] = $row;
		}

		foreach ($rows as $row)
		{
			if (! $row["size"])
			{
				$size = $min;
			}
			else
			{
				$size = $maxr - $minr < 1
				        ? $min
				        : ($max - $min) * ($row["size"] - $minr) / ($maxr - $minr) + $min;
			}
			$mes .= '<nobr><a href="javascript:void(0)" class="tags_add" style="font-size: '.$size.'em;">'
			        .$row["name"].'</a></nobr> ';
		}
		$mes = ($mes ? '<div style="float:right;padding: 0px 0px 5px 5px"><a href="javascript:void(0)" onclick="$(\'.tags_search\').hide()">x</a></div>' : '')
		       .$mes;
		$this->result["data"] = $mes;
		$this->send_json();
		return TRUE;
	}

	/**
	 * Отдает ответ Ajax
	 * 
	 * @return void
	 */
	private function send_json()
	{
		if ($this->result)
		{
			include_once(ABSOLUTE_PATH.'plugins/json.php');
			echo to_json($this->result);
			exit;
		}
	}

	/**
	 * Возвращает номер элемента, к которому подключается фотография или тэг
	 * 
	 * @return integer
	 */
	private function get_id_element()
	{
		if (! empty($_POST["id"]))
		{
			return intval($_POST["id"]);
		}
		else
		{
			$names  = array();
			$values = array();
			if ($this->config_module["parent"])
			{
				$names[]  = "parent_id";
				$values[] = "'".$this->diafan->parent."'";
			}
			if ($this->config_module["element"])
			{
				$names[]  = "cat_id";
				$values[] = "'".$this->diafan->cat."'";
			}
			if ($this->config_module["element_site"])
			{
				$names[]  = "site_id";
				$values[] = "'".$this->diafan->site."'";
			}

			DB::query("INSERT INTO {".$this->table."} (".implode(', ', $names).") VALUES (".implode(', ', $values).")");

			return DB::next_id($this->table);
		}
	}
}