<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.5
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Votes_ajax
 *
 * Обработка запроса при голосовании
 */
class Votes_ajax
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var array результат выполнения запроса
	 */
	private $result;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
	}

	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (! empty($_POST['module']) && $_POST['module'] == 'votes' && ! empty($_POST['question']))
		{
			if (empty($_POST["result"]))
			{
				if ($this->check_element())
					return TRUE;

				$this->check_security_user();
				$this->check_captcha();
				$this->check_log();

				if ($this->send_errors())
					return TRUE;
	
				DB::query("UPDATE {votes} SET count_votes=count_votes+1 WHERE id='%d' AND cat_id='%d'", $_POST["answer"], $_POST["question"]);
			}
			else
			{
				$view_result["result"] = TRUE;
			}

			$view_result["question_id"] = intval($_POST["question"]);
			if (empty($_POST["result"]) || $_POST["result"] == 1)
			{
				$sum_votes = DB::query_result("SELECT SUM(count_votes) FROM {votes} WHERE trash='0' AND cat_id='%d'", $_POST["question"]);

				$result = DB::query("SELECT count_votes, [name] FROM {votes} WHERE trash='0' AND cat_id='%d' ORDER BY id ASC", $_POST["question"]); 
				while ($row = DB::fetch_array($result))
				{
					$row_answer["count"]   = $row["count_votes"];
					$row_answer["persent"] = $sum_votes ? round($row["count_votes"] / $sum_votes * 100) : 0;
					$row_answer["name"] = $row["name"];
					$rows[] = $row_answer;
				}
				$view_result["answers"]  = $rows;
				$view_result["summ"] = $sum_votes;

				$this->result["data"] = $this->diafan->_tpl->get('answers', 'votes', $view_result);
			}
			else
			{
				$result = DB::query("SELECT id, [name] FROM {votes} WHERE trash='0' AND cat_id='%d' ORDER BY id ASC", $_POST["question"]); 
				while ($row = DB::fetch_array($result))
				{
					$row_answer = array();
					$row_answer["id"] = $row["id"];
					$row_answer["name"] = $row["name"];
					$view_result["answers"][] = $row_answer;
				}
				$view_result["captcha"] = '';
				if ($this->diafan->configmodules('security', 'votes') == 2)
				{
					$view_result["captcha"] = $this->diafan->_captcha->get("votes".$row["id"], "");
				}
	
				$this->result["data"] = $this->diafan->_tpl->get('form', 'votes', $view_result);	
			}
			$this->result["target"] = '#votes'.$_POST["question"];
		}
		return $this->send_errors();
	}

	/**
	 * Проверяет существует ли вопрос и ответ в базе
	 * 
	 * @return boolean
	 */
	private function check_element()
	{
		if (empty($_POST['answer']))
		{
			return TRUE;
		}
		if (! DB::query_result("SELECT id FROM {votes_category} WHERE id=%d", $_POST["question"]))
		{
			return TRUE;
		}
		if (! DB::query_result("SELECT id FROM {votes} WHERE id=%d AND cat_id=%d", $_POST['answer'], $_POST["question"]))
		{
			return TRUE;
		}
	}

	/**
	 * Проверяет правильность каптчи
	 * 
	 * @return boolean
	 */
	private function check_captcha()
	{
		$this->result["code"] = '';
		if ($this->diafan->configmodules('security', 'votes') == 2)
		{
			$mes = $this->diafan->_captcha->is_right('votes'.$_POST["question"]);
			$this->result["code"] = $this->diafan->_captcha->get('votes'.$_POST["question"], '', TRUE);				
			if ($mes)
			{
				$this->result["errors"]["code"] = $mes;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Проверяет авторизацию, если задан доступ только для пользователей
	 * 
	 * @return boolean true
	 */
	private function check_security_user()
	{
		if ($this->diafan->configmodules('security_user', 'votes') && ! $this->diafan->_user->id)
		{
			$this->result["errors"][0] = _LANG_VOTING_CLOSE_UNREGISTRED;
		}
		return TRUE;
	}

	/**
	 * Проверяет повторное голосование
	 * 
	 * @return boolean true
	 */
	private function check_log()
	{
		if ($this->diafan->configmodules('security', 'votes') == 3)
		{
			if (DB::query_result("SELECT COUNT(id) FROM {log_note} WHERE session_id='%s' AND element_id='%d' AND include_name='votes'",
								$this->diafan->configmodules('security_user', 'votes') ? $this->diafan->_user->id : session_id(), $_POST["question"]))
			{
				$this->result["errors"][0] = _LANG_VOTE_SECOND;
			}
			else
			{
				DB::query("INSERT INTO {log_note} (include_name, element_id, note, created, ip, session_id)"
				          ." VALUES ('votes', '%d', '%d', %d, '%s', '%s')",
				          $_POST["question"],
				          $_POST["answer"],
				          time(),
				          getenv('REMOTE_ADDR'),
				          $this->diafan->configmodules('security_user', 'votes') ? $this->diafan->_user->id : session_id()
				         );
			}
		}
		elseif ($this->diafan->configmodules('security', 'votes') == 4)
		{
			if (! empty($_SESSION["votes"][$_POST["question"]]))
			{
				$this->result["errors"][0] = _LANG_VOTE_SECOND;
			}
			$_SESSION["votes"][$_POST["question"]] = 1;
		}
		return TRUE;
	}

	/**
	 * Отправляет несколько ошибок
	 *
	 * @return boolean
	 */
	private function send_errors()
	{
		if (! empty($this->result["errors"]) || ! empty($this->result["data"]))
		{
			if (! empty($_POST['ajax']))
			{
				include_once(ABSOLUTE_PATH.'plugins/json.php');
				echo to_json($this->result);
			}
			else
			{
				$query = '';
				if (! empty($this->result["errors"]))
				{
					foreach ($this->result["errors"] as $field => $mes)
					{
						if (empty($_POST["update"]))
						{
							$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
						}
					}
				}
				$this->diafan->redirect($this->diafan->_route->current_link().($query ? '?error=votes'.$_POST["question"].$query : ''));
			}
			return TRUE;
		}
		return FALSE;
	}
}